#include "version.h"

#include <qcheckbox.h>
#include <qdatetime.h>
#include <qdir.h>
#include <qdragobject.h>
#include <qfileinfo.h>
#include <qheader.h>
#include <qmessagebox.h>
#include <qpixmap.h>
#include <qstatusbar.h>
#include <qtextcodec.h>
#include <qtextedit.h>
#include <qtextstream.h>
#include <qtooltip.h>

#include <kaboutapplication.h>
#include <kcmdlineargs.h>
#include <kdirselectdialog.h>
#include <kfiledialog.h>
#include <kmdcodec.h>
#include <kprogress.h>
#include <kiconeffect.h>
#include <kurl.h>

#include <fileref.h>
#include <tag.h>

#define CURRENT_PLAYLIST collection.getPlaylist(tabPlaylist->currentPageIndex())
#define CURRENT_TRACK lstPlaylist->currentItem()
#define WAIT_CURSOR setCursor(Qt::WaitCursor);
#define ARROW_CURSOR setCursor(Qt::ArrowCursor);

void frmMain::init()
{
	srand((unsigned int)(QTime::currentTime(Qt::LocalTime).msec() * QTime::currentTime(Qt::LocalTime).second()));

	grpControl->setTitle(i18n("Control"));
	grpVolume->setTitle(i18n("Volume"));
	grpPlaylist->setTitle(i18n("Playlist"));
	grpTrack->setTitle(i18n("Track"));

	QToolTip::add(pshNewPlaylist, i18n("Add playlist"));
	QToolTip::add(pshUp, i18n("Move up"));
	QToolTip::add(pshDown, i18n("Move down"));
	QToolTip::add(pshGoToCurrent, i18n("Go to current"));
	QToolTip::add(pshRemove, i18n("Remove"));
	QToolTip::add(pshDeletePlaylist, i18n("Delete playlist"));
	QToolTip::add(pshClearList, i18n("Clear playlist"));

	lstPlaylist->setSorting(-1);
	lstPlaylist->setResizeMode(QListView::NoColumn);
	lstPlaylist->setDropVisualizer(false);
	lstPlaylist->setAllColumnsShowFocus(true);
	lstPlaylist->header()->setResizeEnabled(true);

	lstPlaylist->addColumn(i18n("Title"), 255);
	lstPlaylist->addColumn(i18n("Album"), 222);
	lstPlaylist->addColumn(i18n("Artist"), 196);
	lstPlaylist->addColumn(i18n("Bitrate"), 106);
	lstPlaylist->addColumn(i18n("Length"), 76);

	QTimer::singleShot(0, this, SLOT(restore()));
}

void frmMain::restore()
{
	qDebug("Welcome to " + (QString)FALF_NAM + " " + (QString)FALF_VER +
		" (using libxine " + player.getXineVer() + ")!");

	systemTray = new KMSystemTray(this, "TrayIcon");
	QPixmap icon;
	KIconLoader * loader = KGlobal::iconLoader();

	pixPlay = loader->loadIcon("player_play", KIcon::Toolbar);
	pixPause = loader->loadIcon("player_pause", KIcon::Toolbar);
	pixStop = loader->loadIcon("player_stop", KIcon::Toolbar);
	pixPlayLoop = loader->loadIcon("reload", KIcon::Toolbar);
	pixPlayOnce = loader->loadIcon("bottom", KIcon::Toolbar);
	pixTrackLoop = loader->loadIcon("lock", KIcon::Toolbar);
	pixRndOn = loader->loadIcon("wizard", KIcon::Toolbar);
	pixRndOff = loader->loadIcon("forward", KIcon::Toolbar);

	popupMenu = systemTray->contextMenu();
	icon = loader->loadIcon("player_playlist", KIcon::Toolbar);
	popupMenu->insertItem("", this, SLOT(onLeft()), CTRL+Key_P, 1);
	popupMenu->insertItem(pixPlay, i18n("Play"), this, SLOT(onAction()), CTRL+Key_A, 0);
	popupMenu->insertItem("", this, SLOT(onStop()), CTRL+Key_H, 3);
	popupMenu->insertItem("", this, SLOT(onRight()), CTRL+Key_N, 2);
	popupMenu->insertSeparator();
	icon = loader->loadIcon("messagebox_info", KIcon::Toolbar);
	popupMenu->insertItem(icon, i18n("About..."), this, SLOT(onAbout()), 0, 5);

	systemTray->setPixmap(pixStop);
	systemTray->setCaption((QString)FALF_NAM);
	systemTray->show();

	pshAction->setPixmap(pixPlay);
	pshLoop->setPixmap(pixPlayLoop);

	contextMenu = new KPopupMenu(lstPlaylist);
	icon = loader->loadIcon("falf", KIcon::Toolbar);
	contextMenu->insertTitle(icon, (QString)FALF_NAM + " " + (QString)FALF_VER);
	icon = loader->loadIcon("sound", KIcon::Toolbar);
	contextMenu->insertItem(icon, i18n("Add file(s)") + "...", this, SLOT(addFiles()), CTRL+Key_O);
	icon = loader->loadIcon("folder", KIcon::Toolbar);
	contextMenu->insertItem(icon, i18n("Add folder") + "...", this, SLOT(addFolders()), CTRL+Key_D);
	icon = loader->loadIcon("cdaudio_unmount", KIcon::Toolbar);
	contextMenu->insertItem(icon, i18n("Add audio CD contents") + "...", this, SLOT(openAudioCD()), CTRL+Key_C);
	contextMenu->insertSeparator();
	icon = loader->loadIcon("filesave", KIcon::Toolbar);
	contextMenu->insertItem(icon, i18n("Save playlist") + "...", this, SLOT(saveM3u()), CTRL+Key_S);
	QPopupMenu * subMenuRemove = new QPopupMenu(contextMenu);
	subMenuRemove->insertItem(i18n("duplicates"), 0, 0);
	subMenuRemove->insertItem(i18n("dead entries"), 1, 1);
	subMenuRemove->insertSeparator();
	subMenuRemove->insertItem(i18n("both kinds"), 3, 3);
	subMenuRemove->setAccel(CTRL+Key_R, 3);
	icon = loader->loadIcon("editdelete", KIcon::Toolbar);
	contextMenu->insertItem(icon, i18n("Remove from playlist"), subMenuRemove);
	connect(subMenuRemove, SIGNAL(activated(int)), this, SLOT(makeUpPlaylist(int)));
	subMenuSortOrder = new QPopupMenu(contextMenu);
	subMenuSortOrderAscending = new QPopupMenu(subMenuSortOrder);
	subMenuSortOrderAscending->insertItem(i18n("title"), 0, 0);
	subMenuSortOrderAscending->insertItem(i18n("album"), 1, 1);
	subMenuSortOrderAscending->insertItem(i18n("artist"), 2, 2);
	subMenuSortOrderAscending->insertItem(i18n("length"), 3, 3);
	subMenuSortOrderDescending = new QPopupMenu(subMenuSortOrder);
	subMenuSortOrderDescending->insertItem(i18n("title"), 4, 0);
	subMenuSortOrderDescending->insertItem(i18n("album"), 5, 1);
	subMenuSortOrderDescending->insertItem(i18n("artist"), 6, 2);
	subMenuSortOrderDescending->insertItem(i18n("length"), 7, 3);
	icon = loader->loadIcon("1uparrow", KIcon::Toolbar);
	subMenuSortOrder->insertItem(icon, i18n("descending by"), subMenuSortOrderDescending);
	icon = loader->loadIcon("1downarrow", KIcon::Toolbar);
	subMenuSortOrder->insertItem(icon, i18n("ascending by"), subMenuSortOrderAscending);
	connect(subMenuSortOrderAscending, SIGNAL(activated(int)), this, SLOT(sortPlaylist(int)));
	connect(subMenuSortOrderDescending, SIGNAL(activated(int)), this, SLOT(sortPlaylist(int)));
	icon = loader->loadIcon("rebuild", KIcon::Toolbar);
	contextMenu->insertItem(icon, i18n("Sort playlist"), subMenuSortOrder);
	contextMenu->insertSeparator();
	QPopupMenu * subMenuSettings = new QPopupMenu(contextMenu);
	icon = loader->loadIcon("edit_user", KIcon::Toolbar);
	subMenuSettings->insertItem(icon, "last.fm...", this, SLOT(editLfm()), CTRL+Key_U);
	dlgLfm.setIcon(icon);
	icon = loader->loadIcon("kcmsound", KIcon::Toolbar);
	subMenuSettings->insertItem(icon, i18n("Sound system") + "...", this, SLOT(onSoundSystem()), CTRL+Key_X);
	dlgSoundSystem.setIcon(icon);
	icon = loader->loadIcon("configure", KIcon::Toolbar);
	dlgPlaylist.setIcon(icon);
	contextMenu->insertItem(icon, i18n("Settings"), subMenuSettings);
	contextMenu->insertSeparator();
	QPopupMenu * trackMenu = new QPopupMenu(contextMenu);
	playlistMenu = new QPopupMenu(contextMenu);
	subMenuSend = new QPopupMenu(trackMenu);
	subMenuCopy = new QPopupMenu(trackMenu);
	subMenuAppend = new QPopupMenu(playlistMenu);
	icon = loader->loadIcon("add", KIcon::Toolbar);
	playlistMenu->insertItem(icon, i18n("Append to playlist"), subMenuAppend);
	icon = loader->loadIcon("edit", KIcon::Toolbar);
	trackMenu->insertItem(icon, i18n("Edit tags") + "...", this, SLOT(editTags()), CTRL+Key_E);
	dlgTags.setIcon(icon);
	icon = loader->loadIcon("contents", KIcon::Toolbar);
	trackMenu->insertItem(icon, i18n("Show lyrics") + "...", this, SLOT(onLyrics()), CTRL+Key_L);
	dlgLyrics.setIcon(icon);
	dlgLyrics.pshDelLyrics->setText(i18n("Download lyrics again"));
	connect(dlgLyrics.pshDelLyrics, SIGNAL(clicked()), this, SLOT(onDelLyrics()));
	connect(dlgLyrics.cmbCodec, SIGNAL(activated(int)), this, SLOT(onCodecChanged(int)));
	icon = loader->loadIcon("editcopy", KIcon::Toolbar);
	trackMenu->insertItem(icon, i18n("Copy on playlist"), subMenuCopy);
	playlistMenu->insertItem(icon, i18n("Duplicate"), this, SLOT(copyPlaylist()), CTRL+Key_V);
	icon = loader->loadIcon("goto", KIcon::Toolbar);
	trackMenu->insertItem(icon, i18n("Send on playlist"), subMenuSend);
	icon = loader->loadIcon("view_tree", KIcon::Toolbar);
	contextMenu->insertItem(icon, i18n("Track"), trackMenu);
	icon = loader->loadIcon("player_playlist", KIcon::Toolbar);
	contextMenu->insertItem(icon, i18n("Playlist"), playlistMenu);
	connect(subMenuSend, SIGNAL(activated(int)), this, SLOT(sendOnPlaylist(int)));
	connect(subMenuCopy, SIGNAL(activated(int)), this, SLOT(copyOnPlaylist(int)));
	connect(subMenuAppend, SIGNAL(activated(int)), this, SLOT(appendPlaylist(int)));
	stopMenu = new QPopupMenu(this);
	stopMenu->insertItem(i18n("now"), 0, 0);
	stopMenu->insertItem(i18n("after track"), 1, 1);
	connect(stopMenu, SIGNAL(activated(int)), this, SLOT(onStopMenu(int)));
	connect(pshStop, SIGNAL(clicked()), this, SLOT(onStop()));
	connect(pshStop, SIGNAL(pressed()), this, SLOT(onStopPressed()));

	lstPlaylist->restoreLayout(KGlobal::config(), "KMListView lstPlaylist");

	headerMenu = new QPopupMenu(lstPlaylist->header());
	for (int col_i = 0 ; col_i != lstPlaylist->columns() ; col_i++)
	{
		QCheckBox * tmp = new QCheckBox(lstPlaylist->columnText(col_i), headerMenu);
		tmp->setChecked((bool)lstPlaylist->columnWidth(col_i));
		if (!tmp->isChecked())
			lstPlaylist->header()->setResizeEnabled(false, col_i);
		headerMenuItems.append(tmp);
		headerMenu->insertItem(tmp, col_i, col_i);
	}
	QPushButton * pshHideHeaderMenu = new QPushButton("OK", headerMenu);
	headerMenu->insertItem(pshHideHeaderMenu, lstPlaylist->columns(), lstPlaylist->columns());
	connect(lstPlaylist->header(), SIGNAL(clicked(int)), this, SLOT(onShowPlaylistHeaderMenu(int)));
	connect(headerMenu, SIGNAL(aboutToHide()), this, SLOT(onPlaylistHeaderMenu()));
	connect(pshHideHeaderMenu, SIGNAL(clicked()), headerMenu, SLOT(hide()));

	homeDir = QDir::homeDirPath();
	QDir dir(homeDir + "/.FALF");
	confFile.setName(dir.path() + "/FALF.conf");
	currentTrackFile.setName(dir.path() + "/track.info");
	lockFile.setName(dir.path() + "/lock");
	tmpFiles.setName(dir.path() + "/tmp_files");
	lyrics.setPath(dir.path() + "/lyrics");
	browseDir = homeDir;
	filePattern = "*.mp3 *.ogg *.mpc *.wav *.cda *.mp4 *.m4a *.wma *.mp2 *.aif *.mpp *.aac *.mod *.flac *.aiff *.speex *.xm *.it *.s3m *.m3u";

	KCmdLineArgs * args = KCmdLineArgs::parsedArgs();
	QString files(QString::null);

	for (int argsCount = 0 ; argsCount != args->count() ; argsCount++)
		files += ("file://" + QString::fromLocal8Bit(args->arg(argsCount)));

	alreadyRunning = false;

	if (lockFile.exists())
	{
		if (files.isEmpty())
		{
			if (QMessageBox::information(this, i18n("Info"),
				(QString)FALF_NAM + " " + i18n("is already running") + "!\n" +
				i18n("Running multiple copies of FALF will damage configuration file.") + "\n" +
				i18n("Do You really wanna run next copy?"),
				QMessageBox::Yes, QMessageBox::No | QMessageBox::Default,
				QMessageBox::NoButton) != QMessageBox::Yes)
			{
				alreadyRunning = true;
				destroy();
				exit(1);
			}
		}
		else
		{
			tmpFiles.open(IO_WriteOnly | IO_Append);
			QTextStream tmp(&tmpFiles);
			tmp.setEncoding(QTextStream::Unicode);
				tmp << files;
			tmpFiles.close();

			alreadyRunning = true;
			destroy();
			exit(0);
		}
	}
	else
	{
		lockFile.open(IO_WriteOnly);
		QTextStream lock(&lockFile);
			lock << QString::null;
		lockFile.close();
	}

	if (!dir.exists())
		QDir().mkdir(dir.path());

	loop = 0;
	rnd = false;
	http = 0;
	lyricsErrStr = i18n("No lyrics found");
	frame = 0;
	desiredVolume = 25;
	sldVol->setValue(25);
	player.setVolume(25);
	statBar = statusBar();
	currentTrack = QString::null;
	action = STOP;
	playTab = 0;
	playIndex = 0;
	lfmOn = false;
	lfmChallenge = 0;
	lfmBuffer = 0;
	lfmAddr = 0;
	lfmServer = 0;
	lfmSubmit = 0;
	stopAfter = false;

	if (confFile.exists())
	{
		frmRestore dlgRestore;
		dlgRestore.prgRestore->setProgress(0);
		dlgRestore.show();

		if (confFile.open(IO_ReadOnly))
		{
			qDebug("Restoring...");

			QTextStream streamIn(&confFile);
			streamIn.setEncoding(QTextStream::Unicode);

			QString buffer = streamIn.readLine();

			int pos = -1;

			pos = buffer.find("</files>");
			dlgRestore.prgRestore->setTotalSteps(buffer.mid(7, pos - 7).toInt());

			buffer = streamIn.readLine();

			pos = buffer.find("</volume>");
			desiredVolume = buffer.mid(8, pos - 8).toInt();

			buffer = streamIn.readLine();

			pos = buffer.find("</x>");
			int x = buffer.mid(3, pos - 3).toInt();

			buffer = streamIn.readLine();

			pos = buffer.find("</y>");
			int y = buffer.mid(3, pos - 3).toInt();

			move(x, y);

			buffer = streamIn.readLine();

			pos = buffer.find("</loop>");
			loop = buffer.mid(6, pos - 6).toShort();

			buffer = streamIn.readLine();

			pos = buffer.find("</rand>");
			rnd = (bool)(buffer.mid(6, pos - 6).toUShort());

			buffer = streamIn.readLine();

			pos = buffer.find("</lfm>");
			lfmOn = (bool)(buffer.mid(5, pos - 5).toUShort());

			buffer = streamIn.readLine();

			pos = buffer.find("</user>");
			lfmUser = buffer.mid(6, pos - 6);

			buffer = streamIn.readLine();

			pos = buffer.find("</pass>");
			lfmPassword = buffer.mid(6, pos - 6);

			while (!confFile.atEnd())
			{
				buffer = streamIn.readLine();

				if (buffer.left(6) == "<page>")
					break;

				int pListPos = buffer.find("</playlist>");
				pListIndex * p = collection.newPlaylist(buffer.mid(10, pListPos - 10), "");
				
				if (!p)
					continue;

				buffer = streamIn.readLine();

				pListPos = buffer.find("</descr>");
				collection.setPlaylistDescr(p, buffer.mid(7, pListPos - 7));

				streamIn.readLine();

				while (((buffer = streamIn.readLine()) != "<end>") &&
					(!confFile.atEnd()) && (!buffer.isEmpty()))
				{
					int filePos = buffer.find("</file>");

					QString path = buffer.mid(7, filePos - 7);
					QString name;

					if (path.left(6) == "cdda:/")
						name = (i18n("Track") + " " + path.right(path.length() - 6));
					else
						name = path.mid(path.findRev("/") + 1, path.length() - 4);

					exTags t = readTags(path);

					collection.add(p, name, path, &t.basic);

					dlgRestore.prgRestore->advance(1);
				}
			}

			pos = buffer.find("</page>");
			playTab = buffer.mid(6, pos - 6).toInt();

			buffer = streamIn.readLine();

			pos = buffer.find("</track>");
			int track = buffer.mid(7, pos - 7).toInt();

			buffer = streamIn.readLine();

			pos = buffer.find("</frame>");
			frame = buffer.mid(7, pos - 7).toUInt();

			buffer = streamIn.readLine();

			pos = buffer.find("</browse>");
			browseDir = buffer.mid(8, pos - 8);

			buffer = streamIn.readLine();

			pos = buffer.find("</cddevice>");
			audioCDDevice = buffer.mid(10, pos - 10);
			
			buffer = streamIn.readLine();

			pos = buffer.find("</codec>");
			dlgLyrics.cmbCodec->setCurrentItem(buffer.mid(7, pos - 7).toInt());

			buffer = streamIn.readLine();

			pos = buffer.find("</sound>");
			dlgSoundSystem.cmbSoundSystem->setCurrentItem(buffer.mid(7, pos - 7).toInt());
			player.setSoundSystem(dlgSoundSystem.cmbSoundSystem->currentText().lower());

			if (collection.getPlaylistCount())
			{
				refreshPlaylist(playTab);

				if (playTab == 0)
					refreshList();
			
				if (track >= 0)
				{
					QListViewItem * i = lstPlaylist->itemAtIndex(track);

					if (i)
						lstPlaylist->setCurrentItem(i);

					playIndex = track;
				}

				if ((frame > 0) && (files.isEmpty()))
				{
					player.setPos(frame);
					QTimer::singleShot(0, this, SLOT(onAction()));
				}
			}

			confFile.close();
		}
	}
	else
		onSoundSystem();

	if (!files.isEmpty())
	{
		collection.newPlaylist(
			QDate::currentDate(Qt::LocalTime).toString() + " " +
			QTime::currentTime(Qt::LocalTime).toString(),
			i18n("Double click to rename playlist"));
		
		frame = 0;
		playIndex = 0;
		playTab = (collection.getPlaylistCount() - 1);
		refreshPlaylist(playTab);
		parse(files);
	}

	loop--;
	onLoop();

	rnd = !rnd;
	onRnd();

	if (!collection.getPlaylistCount())
	{
		collection.newPlaylist(i18n("Unnamed Playlist"), i18n("Double click to rename playlist"));
		refreshPlaylist(0);
	}

	setAcceptDrops(true);

	sldVol->setOrientation(Qt::Horizontal);
	sldVol->setPageStep(5);
	sldVol->setValue(desiredVolume);

	connect(sldVol, SIGNAL(valueChanged(int)), this, SLOT(onVolume()));
	
	connect(lstPlaylist, SIGNAL(dropped(QDropEvent *, QListViewItem *)), this, SLOT(fileDropped(QDropEvent *, QListViewItem *)));
	connect(lstPlaylist, SIGNAL(doubleClicked(QListViewItem *)), this, SLOT(onPlaylist()));
	connect(lstPlaylist, SIGNAL(mouseButtonReleased(const QPoint &)), this, SLOT(onMouseButtonReleased(const QPoint &)));
	connect(lstPlaylist, SIGNAL(keyPressed(QKeyEvent *)), this, SLOT(onKeyPressed(QKeyEvent *)));
	connect(lstPlaylist, SIGNAL(rightMouseButtonClicked(const QPoint &)), this, SLOT(showContextMenu(const QPoint &)));

	if (player.getPos())
	{
		player.setVolume(0);
		connect(&volumeTimer, SIGNAL(timeout()), this, SLOT(restoreVolume()));
		volumeTimer.start(500);
	}
	else
		onVolume();

	connect(&playTimer, SIGNAL(timeout()), this, SLOT(onPlayTimer()));
	connect(&searchTimer, SIGNAL(timeout()), this, SLOT(onSearchTimer()));
	connect(&lfmTimer, SIGNAL(timeout()), this, SLOT(lastFmQueue()));
	connect(&tmpFilesTimer, SIGNAL(timeout()), this, SLOT(addTmpFiles()));

	tmpFilesTimer.start(3000);

	connect(systemTray, SIGNAL(quitSelected()), this, SLOT(onQuitSelected()));
	connect(systemTray, SIGNAL(wheelMoved(QWheelEvent *)), this, SLOT(scrollVol(QWheelEvent *)));

	icon = loader->loadIcon("view_remove", KIcon::Toolbar);
	pshClearList->setPixmap(icon);
	icon = loader->loadIcon("tab_remove", KIcon::Toolbar);
	pshDeletePlaylist->setPixmap(icon);
	icon = loader->loadIcon("player_start", KIcon::Toolbar);
	pshLeft->setPixmap(icon);
	popupMenu->changeItem(1, icon, i18n("Previous"));
	icon = loader->loadIcon("player_end", KIcon::Toolbar);
	pshRight->setPixmap(icon);
	popupMenu->changeItem(2, icon, i18n("Next"));
	icon = loader->loadIcon("locationbar_erase", KIcon::Toolbar);
	dlgPlaylist.pshClearTxtPlaylist->setPixmap(icon);
	dlgPlaylist.pshClearTxtDescription->setPixmap(icon);
	icon = loader->loadIcon("add", KIcon::Toolbar);
	pshNewPlaylist->setPixmap(icon);	
	icon = loader->loadIcon("kmix", KIcon::Toolbar);
	pixVol->setPixmap(icon);
	icon = loader->loadIcon("up", KIcon::Toolbar);
	pshUp->setPixmap(icon);
	icon = loader->loadIcon("down", KIcon::Toolbar);
	pshDown->setPixmap(icon);
	icon = loader->loadIcon("redo", KIcon::Toolbar);
	pshGoToCurrent->setPixmap(icon);
	icon = loader->loadIcon("remove", KIcon::Toolbar);
	pshRemove->setPixmap(icon);
	pshStop->setPixmap(pixStop);
	popupMenu->changeItem(3, pixStop, i18n("Stop"));
	popupMenu->setItemEnabled(3, false);

	tabPlaylist->setTabReorderingEnabled(true);
	
	setTip();

	txtSearch->hide();

	if (lfmOn)
		lastFmHandShake();
}

void frmMain::destroy()
{
	player.termThread();
	player.wait();

	if (!alreadyRunning)
	{
		currentTrackFile.remove();
		lockFile.remove();

		qDebug("Storing...");
		store();

		qDebug("Graceful shutdown...");
	}
	else
		qDebug("Aborted!");
}

void frmMain::dragEnterEvent(QDragEnterEvent * event)
{
	event->accept(QTextDrag::canDecode(event));
}

void frmMain::dropEvent(QDropEvent * event)
{
	QString text;

	if (QTextDrag::decode(event, text))
		parse(text);
}

void frmMain::onNewPlaylist()
{
	dlgPlaylist.ok = false;
	dlgPlaylist.txtPlaylist->setText(i18n("Unnamed Playlist"));
	dlgPlaylist.txtDescription->setText("");
	dlgPlaylist.txtPlaylist->setFocus();

	dlgPlaylist.exec();

	if (dlgPlaylist.ok)
	{
		if (!dlgPlaylist.txtPlaylist->text().isEmpty())
			collection.newPlaylist(dlgPlaylist.txtPlaylist->text(), dlgPlaylist.txtDescription->text());
		else
			collection.newPlaylist(i18n("Unnamed Playlist"), dlgPlaylist.txtDescription->text());
		refreshPlaylist(tabPlaylist->count());
	}
}

void frmMain::refreshPlaylist(const int & index)
{
	QWidget * w = 0;

	tabPlaylist->disconnect();

	while (tabPlaylist->count())
	{
		w = tabPlaylist->currentPage();
		tabPlaylist->removePage(w);
		delete w;
	}

	for (int i = 0 ; i != collection.getPlaylistCount() ; i++)
	{
		w = new QWidget;
		
		pListIndex * p = collection.getPlaylist(i);

		if (p)
			tabPlaylist->addTab(w, collection.getPlaylistName(p));
	}

	connect(tabPlaylist, SIGNAL(currentChanged(QWidget*)), this, SLOT(onTabChanged()));
	connect(tabPlaylist, SIGNAL(testCanDecode(const QDragMoveEvent *, bool &)),
		this, SLOT(onTestCanDecode(const QDragMoveEvent *, bool &)));
	connect(tabPlaylist, SIGNAL(receivedDropEvent(QWidget *, QDropEvent *)),
		this, SLOT(onReceivedDropEvent(QWidget *, QDropEvent *)));
	connect(tabPlaylist, SIGNAL(initiateDrag(QWidget *)), this, SLOT(onInitiateDrag(QWidget *)));
	connect(tabPlaylist, SIGNAL(mouseDoubleClick(QWidget *)), this, SLOT(onTabDblClicked(QWidget *)));
	
	if (index != tabPlaylist->currentPageIndex())
		tabPlaylist->setCurrentPage(index);

	setTabTip();
}

void frmMain::setTabTip()
{
	pListIndex * p = 0;
	QWidget * w = 0;

	for (int i = 0 ; i != collection.getPlaylistCount() ; i++)
	{
		p = collection.getPlaylist(i);

		if (!p)
			return;

		w = tabPlaylist->page(i);

		QString s;
		s += "<b>" + collection.getPlaylistName(p) + "</b>";
		QString descr = collection.getPlaylistDescr(p);
		if (!descr.isEmpty())
		{
			s += "<br>";
			s += "<i>" + descr + "</i>";
		}
		s += "<br><br>";
		QString len = QString("%1:%2:%3").arg(p->len.h, 2).arg(p->len.m, 2).arg(p->len.s, 2);
		len.replace(" ", "0");
		s += QString::number(p->count) + " " + i18n("track(s)") + " (" + len + ")";
	
		tabPlaylist->setTabToolTip(w, s);
	}
}

void frmMain::refreshList()
{
	pListIndex * p = collection.getPlaylist(tabPlaylist->currentPageIndex());

	QListViewItem * item = lstPlaylist->currentItem();

	int curSel = -1;

	if (item)
		curSel = lstPlaylist->itemIndex(item);

	lstPlaylist->clear();

	if ((p) && (p->list))
	{
		int i = 0;
		pList * l;

		while ((l = collection.get(p, i++)))
		{
			QListViewItem * newItem = NULL;

			QString title = l->fileTags.tagTitle;

			if (title.isEmpty())
				title = l->trackName;

			newItem = new KListViewItem(lstPlaylist, lstPlaylist->lastItem(),
				title,
				l->fileTags.tagAlbum,
				l->fileTags.tagArtist,
				(l->fileTags.tagBitrate + " kbps"),
				l->fileTags.tagLength);

			newItem->setDropEnabled(true);
			newItem->setDragEnabled(true);
		}

		lstPlaylist->setSelected(lstPlaylist->itemAtIndex(curSel), true);
	}
	
	markCurrent();
	onOff();

	setStatBarMsg();
}

void frmMain::onDeletePlaylist()
{
	if (collection.getPlaylistCount() == 1)
		return;

	int index = tabPlaylist->currentPageIndex();

	QMessageBox msg(i18n("Question"), i18n("Do You really wanna delete playlist") + " \"" + tabPlaylist->label(index).remove("&") + "\"?", QMessageBox::Warning,
		QMessageBox::Yes, QMessageBox::No | QMessageBox::Default, QMessageBox::NoButton);

	if (msg.exec() != QMessageBox::Yes)
		return;

	QWidget * w = tabPlaylist->currentPage();
	pListIndex * p = collection.getPlaylist(tabPlaylist->indexOf(w));

	if (!p)
		return;

	clearStop(true);

	collection.delPlaylist(p);
	tabPlaylist->removePage(w);
	delete w;

	tabPlaylist->setCurrentPage(index - 1);

	setTip();
}

void frmMain::onTabChanged()
{
	txtSearch->hide();

	subMenuSend->clear();
	subMenuCopy->clear();
	subMenuAppend->clear();

	for (int i = 0 ; i != collection.getPlaylistCount() ; i++)
	{
		pListIndex * p = collection.getPlaylist(i);

		if (p)
		{
			if (i != tabPlaylist->currentPageIndex())
			{
				subMenuSend->insertItem(p->name, i, i);
				subMenuCopy->insertItem(p->name, i, i);
				subMenuAppend->insertItem(p->name, i, i);
			}
			else
			{
				refreshList();

				subMenuSend->insertSeparator();
				subMenuCopy->insertSeparator();
				subMenuAppend->insertSeparator();
			}
		}

	}

	lstPlaylist->clearSelection();

	QListViewItem * i = lstPlaylist->currentItem();

	if (!i)
		return;

	if (tabPlaylist->currentPageIndex() == playTab)
		lstPlaylist->setSelected(lstPlaylist->itemAtIndex(playIndex), true);
	else
	{
		QListViewItem * i = lstPlaylist->firstChild();
		if (i)
			lstPlaylist->setSelected(i, true);
	}

	ensureVisible();
	lstPlaylist->setFocus();
}

void frmMain::onClearList()
{
	unsigned int index = tabPlaylist->currentPageIndex();

	QMessageBox msg(i18n("Question"), i18n("Do You really wanna delete all files from playlist") + " \"" + tabPlaylist->label(index).remove("&") + "\"?", QMessageBox::Warning,
		QMessageBox::Yes, QMessageBox::No | QMessageBox::Default, QMessageBox::NoButton);

	if (msg.exec() != QMessageBox::Yes)
		return;

	pListIndex * p = collection.getPlaylist(index);

	if (!p)
		return;

	lstPlaylist->clear();

	pshClearList->setEnabled(false);

	clearStop();

	collection.del(p, -1);

	refreshList();
	setTabTip();
	setTip();
}

void frmMain::onAction()
{
	pListIndex * p = NULL;
	pList * l = NULL;

	QListViewItem * item = lstPlaylist->currentItem();

	if ((!item) && (action != PLAY))
		return;

	bool enable = true;

	lfmTimer.stop();

	switch (action)
	{
		case PAUSE:
		case STOP:
			playTab = tabPlaylist->currentPageIndex();
			playIndex = lstPlaylist->itemIndex(item);
			lstPlaylist->setSelected(item, true);

			p = collection.getPlaylist(playTab);

			if (!p)
				return;

			l = collection.get(p, playIndex);

			if (!l)
				return;

			if ((rnd) && (action != PAUSE))
				randList.addIndex(playIndex);
		
			player.setTrack(l->filePath);
			player.wakeThread();

			if (!frame)
				player.setPos(sldPos->value());
			else
			{
				player.setPos(frame);
				frame = 0;
			}
			
			if (!player.running())
				player.start(QThread::TimeCriticalPriority);

			playTimer.start(1000, false);
			sldPos->setEnabled(true);
			
			pshAction->setPixmap(pixPause);
			systemTray->setPixmap(pixPlay);
			popupMenu->changeItem(0, pixPause, i18n("Pause"));
			
			if (dlgLyrics.isVisible())
				onLyrics();

			if ((lfmOn) && (action != PAUSE))
			{
				if (l->fileTags.tagLengthSeconds >= 30)
					lfmTimer.start((l->fileTags.tagLengthSeconds * 900), true);
			}

			action = PLAY;
		break;
		case PLAY:
			player.termThread();
			player.wait();
			playTimer.stop();
			tabPlaylist->setCurrentPage(playTab);
			pshAction->setPixmap(pixPlay);
			systemTray->setPixmap(pixPause);
			popupMenu->changeItem(0, pixPlay, i18n("Play"));
			action = PAUSE;
			enable = false;
		break;
	}

	lstPlaylist->setEnabled(enable);
	tabPlaylist->setEnabled(enable);

	if ((tabPlaylist->count() != 1) && (action != PAUSE))
		pshDeletePlaylist->setEnabled(true);
	else
		pshDeletePlaylist->setEnabled(false);

	pshClearList->setEnabled(enable);
	pshNewPlaylist->setEnabled(enable);
	//pshRenamePlaylist->setEnabled(enable);
	pshUp->setEnabled(enable);
	pshDown->setEnabled(enable);
	pshRemove->setEnabled(enable);
	pshStop->setEnabled(enable);
	popupMenu->setItemEnabled(1, enable);
	popupMenu->setItemEnabled(2, enable);
	popupMenu->setItemEnabled(3, enable);
	pshLeft->setEnabled(enable);
	pshRight->setEnabled(enable);

	setTip();
	markCurrent();
	ensureVisible();
}

void frmMain::onVolume()
{
	volumeTimer.stop();
	volumeTimer.disconnect();
	player.setVolume(sldVol->value());
}

void frmMain::onPlayTimer()
{
	sldPos->setMinValue(0);
	sldPos->setMaxValue(player.getLength()); // doesn't work without it ;[
	sldPos->setValue(player.getPos());

	if (!player.running())
	{
		playTimer.stop();

		tabPlaylist->setCurrentPage(playTab);

		if (stopAfter)
		{
			onStop();
			return;
		}

		if (loop == 2)
		{
			action = STOP;
			lstPlaylist->setCurrentItem(lstPlaylist->itemAtIndex(playIndex));
			onAction();
			return;
		}

		if (!rnd)
			onRight();
		else
			random();
	}
}

void frmMain::onSldPosPressed()
{
	currentTrack = player.getTrack();
	lfmTimer.stop();
	playTimer.stop();
	player.termThread();
	player.wait();
}

void frmMain::onSldPosReleased()
{
	player.setTrack(currentTrack);
	player.wakeThread();
	player.start(QThread::TimeCriticalPriority);
	playTimer.start(1000, false);
}

void frmMain::onSldPos()
{
	player.setPos(sldPos->value());
}

void frmMain::onLeft()
{
	tabPlaylist->setCurrentPage(playTab);

	QListViewItem * item = lstPlaylist->currentItem();

	if (!item)
		return;

	int i = lstPlaylist->itemIndex(item);

	switch (action)
	{
		case PLAY:
			player.termThread();
			player.wait();
			playTimer.stop();
			sldPos->setValue(0);
			action = STOP;

			if (rnd)
				random();
			else
			{
				if (i > 0)
				{
					lstPlaylist->setSelected(lstPlaylist->itemAtIndex(i - 1), true);
					onAction();
				}
				else
				{
					sldPos->setEnabled(false);
					pshAction->setPixmap(pixPlay);
					setTip();
				}
			}
		break;
		case STOP:
		break;
	}

	onOff();
	ensureVisible();
}

void frmMain::onRight()
{
	tabPlaylist->setCurrentPage(playTab);

	QListViewItem * item = lstPlaylist->currentItem();

	if (!item)
		return;

	int i = lstPlaylist->itemIndex(item);

	switch (action)
	{
		case PLAY:
			player.termThread();
			player.wait();
			playTimer.stop();
			sldPos->setValue(0);
			action = STOP;
			
			if (rnd)
				random();
			else
			{
				if (i < lstPlaylist->childCount() - 1)
				{
					lstPlaylist->setSelected(lstPlaylist->itemAtIndex(i + 1), true);
					onAction();
				}
				else
				{
					sldPos->setEnabled(false);
					pshAction->setPixmap(pixPlay);
					setTip();
			
					if (!loop)
					{
						pshStop->setEnabled(false);
						popupMenu->setItemEnabled(3, false);
						systemTray->setPixmap(pixStop);
						popupMenu->changeItem(0, pixPlay, i18n("Play"));
					}
					else
					{
						QListViewItem * first = lstPlaylist->firstChild();

						if (first)
						{
							lstPlaylist->setSelected(first, true);
							onAction();
						}
					}
				}
			}
		break;
		case STOP:
		break;
	}

	onOff();
	ensureVisible();
}

void frmMain::onPlaylist()
{
	QListViewItem * i = lstPlaylist->currentItem();

	if (!i)
		return;

	player.termThread();
	player.wait();
	playTimer.stop();
	sldPos->setValue(0);
	action = STOP;

	playTab = tabPlaylist->currentPageIndex();
	playIndex = lstPlaylist->itemIndex(i);

	randList.remove();

	onAction();
}

void frmMain::onUp()
{
	pListIndex * p = collection.getPlaylist(tabPlaylist->currentPageIndex());

	if (!p)
		return;

	QListViewItem * item = lstPlaylist->currentItem();

	int i = -1;

	if (item)
		i = lstPlaylist->itemIndex(item);

	if (i <= 0)
		return;

	collection.up(p, i);
	randMove(i, i - 1);
	refreshList();

	lstPlaylist->setSelected(lstPlaylist->itemAtIndex(i - 1), true);

	ensureVisible();
}

void frmMain::onDown()
{
	pListIndex * p = collection.getPlaylist(tabPlaylist->currentPageIndex());

	if (!p)
		return;

	QListViewItem * item = lstPlaylist->currentItem();

	int i = -1;

	if (item)
		i = lstPlaylist->itemIndex(item);

	if (i + 1 >= lstPlaylist->childCount())
		return;

	collection.down(p, i);
	randMove(i, i + 1);
	refreshList();

	lstPlaylist->setSelected(lstPlaylist->itemAtIndex(i + 1), true);

	ensureVisible();
}

void frmMain::onRemove()
{
	pListIndex * p = collection.getPlaylist(tabPlaylist->currentPageIndex());

	if (!p)
		return;

	QListViewItem * item = lstPlaylist->currentItem();

	int i = -1;

	if (!item)
		return;

	i = lstPlaylist->itemIndex(item);

	bool wasPlaying = false;

	if (tabPlaylist->currentPageIndex() == playTab)
	{
		if (i == playIndex)
		{
			if (player.running())
				wasPlaying = true;
			action = STOP;
			playTimer.stop();
			player.termThread();
			player.wait();
		}
		else
		{
			if (playIndex > i)
				playIndex--;
		}

		if (!randList.isEmpty())
		{
			if (randList.searchForIndex(i))
				randList.deleteIndex(i);

			for (int c = (i + 1) ; c < lstPlaylist->childCount() ; c++)
			{
				if (randList.searchForIndex(c))
				{
					randList.deleteIndex(c);
					randList.addIndex(c - 1);
				}
			}
		}
	}

	collection.del(p, i);
	refreshList();
	setTabTip();

	if (i + 1 <= lstPlaylist->childCount())
		lstPlaylist->setSelected(lstPlaylist->itemAtIndex(i), true);
	else
		lstPlaylist->setSelected(lstPlaylist->itemAtIndex(0), true);

	if (!player.running())
	{
		sldPos->setValue(0);

		if (lstPlaylist->childCount() == 0)
			sldPos->setEnabled(false);

		if (wasPlaying)
		{
			if (rnd)
				random();
			else
				onAction();
		}
	}

	ensureVisible();
}

void frmMain::onGoToCurrent()
{
	tabPlaylist->setCurrentPage(playTab);
}

void frmMain::onQuitSelected()
{
	unsigned short int count = queue.count();
	if (count > 0)
	{
		QMessageBox msg(i18n("Question"), QString::number(count) + " " + 
			i18n("track(s) remain(s) in last.fm's queue.") + "\n" +
			i18n("Do You really wanna quit?"), QMessageBox::Warning,
			QMessageBox::Yes, QMessageBox::No | QMessageBox::Default, QMessageBox::NoButton);

		if (msg.exec() != QMessageBox::Yes)
			return;
	}

	dlgLyrics.hide();
	systemTray->hide();
}

void frmMain::onAbout()
{
	KAboutApplication aboutApp;
	aboutApp.exec();
}

void frmMain::onLoop()
{
	if (loop == 2)
		loop = 0;
	else
		loop++;

	switch (loop)
	{
		case 0:
			pshLoop->setPixmap(pixPlayOnce);
			QToolTip::add(pshLoop, i18n("Repeat playlist"));
		break;
		case 1:
			pshLoop->setPixmap(pixPlayLoop);
			QToolTip::add(pshLoop, i18n("Repeat track"));
		break;
		case 2:
			pshLoop->setPixmap(pixTrackLoop);
			QToolTip::add(pshLoop, i18n("Don't repeat playlist"));
		break;
	}
}

QString frmMain::getTimeAsString(const int & time)
{
	int seconds = time % 60;
	int minutes = (time - seconds) / 60;
	
	QString min = QString::number(minutes);
	QString sec = QString::number(seconds);

	if (min.length() == 1)
		min = ("0" + min);

	if (sec.length() == 1)
		sec = ("0" + sec);

	return (min + ":" + sec);
}

exTags frmMain::readTags(const QString & file)
{
	exTags ft;

	if (file.left(6) != "cdda:/")
	{
		QString crap("! WWW.POLSKIE-MP3.TK !");

		TagLib::FileRef f((const char *)file.local8Bit());

		if (!f.isNull())
		{
			if (TagLib::Tag * tag = f.tag())
			{
				ft.basic.tagTitle = TStringToQString(tag->title());
				ft.basic.tagAlbum = TStringToQString(tag->album());
				ft.basic.tagArtist = TStringToQString(tag->artist());

				ft.basic.tagTitle.remove(crap);
				ft.basic.tagTitle.remove(crap.lower());
				ft.basic.tagAlbum.remove(crap);
				ft.basic.tagAlbum.remove(crap.lower());
				ft.basic.tagArtist.remove(crap);
				ft.basic.tagArtist.remove(crap.lower());

				ft.basic.tagTitle = ft.basic.tagTitle.simplifyWhiteSpace();
				ft.basic.tagAlbum = ft.basic.tagAlbum.simplifyWhiteSpace();
				ft.basic.tagArtist = ft.basic.tagArtist.simplifyWhiteSpace();

				ft.tagYear = QString::number(tag->year());
				ft.tagComment = TStringToQString(tag->comment());
				ft.tagTrack = QString::number(tag->track());
				ft.tagGenre = TStringToQString(tag->genre());
			}

			if (TagLib::AudioProperties * properties = f.audioProperties())
			{
				int brate = properties->bitrate();

				ft.basic.tagBitrate = (QString::number(brate));
				ft.basic.tagLength = getTimeAsString(properties->length());
				ft.basic.tagLengthSeconds = properties->length();
			}
		}
	}
	else
	{
		static xine_t * xine = (xine_t *)player.getXineHandle();
		
		if (!xine)
			return ft;

		xine_stream_t * tmp = xine_stream_new(xine, NULL, NULL);
		
		if (xine_open(tmp, QFile::encodeName(file)))
		{
			ft.basic.tagTitle = QString::fromUtf8(xine_get_meta_info(tmp, XINE_META_INFO_TITLE));

			if (!ft.basic.tagTitle.isEmpty())
			{
				ft.basic.tagAlbum =
					QString::fromUtf8(xine_get_meta_info(tmp, XINE_META_INFO_ALBUM));
                		ft.basic.tagArtist =
					QString::fromUtf8(xine_get_meta_info(tmp, XINE_META_INFO_ARTIST));
			}

			int sampleRate = xine_get_stream_info(tmp, XINE_STREAM_INFO_AUDIO_SAMPLERATE);
			int bitsPerSample = xine_get_stream_info(tmp, XINE_STREAM_INFO_AUDIO_BITS);
			int numberOfChannels = xine_get_stream_info(tmp, XINE_STREAM_INFO_AUDIO_CHANNELS);
			int brate = (sampleRate * bitsPerSample * numberOfChannels) / 1000;

			int length = 0;
			xine_get_pos_length(tmp, NULL, NULL, &length);

			length /= 1000;
			
			ft.basic.tagBitrate = QString::number(brate);
			ft.basic.tagLength = getTimeAsString(length);
			ft.basic.tagLengthSeconds = length;

			xine_close(tmp);
        	}

		xine_dispose(tmp);
	}

	return ft;
}

void frmMain::editTags()
{
	if (lstPlaylist->childCount() == 0)
	{
		QMessageBox::information(this, i18n("Info"), i18n("First select some file!"));
		return;
	}

	pListIndex * p = collection.getPlaylist(tabPlaylist->currentPageIndex());

	if (!p)
		return;

	QListViewItem * item = lstPlaylist->currentItem();

	int i = -1;

	if (!item)
		return;

	i = lstPlaylist->itemIndex(item);

	pList * l = collection.get(p, i);

	if (!l)
		return;

	dlgTags.txtTitle->setText(l->fileTags.tagTitle);
	dlgTags.txtAlbum->setText(l->fileTags.tagAlbum);
	dlgTags.txtArtist->setText(l->fileTags.tagArtist);

	exTags eT = readTags(l->filePath);

	dlgTags.txtYear->setText(eT.tagYear);
	dlgTags.txtComment->setText(eT.tagComment);
	dlgTags.txtTrack->setText(eT.tagTrack);
	dlgTags.txtGenre->setText(eT.tagGenre);

	dlgTags.txtTitle->setFocus();

	dlgTags.ok = false;
	dlgTags.reload = false;

	dlgTags.exec();

	if (dlgTags.ok)
	{
		writeTags(l->filePath,
			dlgTags.txtTitle->text(),
			dlgTags.txtAlbum->text(),
			dlgTags.txtArtist->text(),
			dlgTags.txtYear->text(),
			dlgTags.txtComment->text(),
			dlgTags.txtTrack->text(),
			dlgTags.txtGenre->text());

		eT = readTags(l->filePath);
		
		l->fileTags.tagTitle = eT.basic.tagTitle;
		l->fileTags.tagAlbum = eT.basic.tagAlbum;
		l->fileTags.tagArtist = eT.basic.tagArtist;

		refreshList();
	}

	if (dlgTags.reload)
	{
		bool changed = false;

		if (dlgTags.next)
		{
			if ((i != -1) && ((i + 1) < lstPlaylist->childCount()))
			{
				lstPlaylist->setSelected(lstPlaylist->itemAtIndex(i + 1), true);
				changed = true;
			}
		}
		else
		{
			if (i > 0)
			{
				lstPlaylist->setSelected(lstPlaylist->itemAtIndex(i - 1), true);
				changed = true;
			}
		}

		ensureVisible();
		
		if (changed)
			editTags();
	}
}

void frmMain::writeTags(const QString & file,
	const QString & title,
	const QString & album,
	const QString & artist,
	const QString & year,
	const QString & comment,
	const QString & track,
	const QString & genre)
{
	tags ft;

	TagLib::FileRef f((const char *)file.local8Bit());

	if (!f.isNull())
	{
		TagLib::Tag * tag = f.tag();
		
		tag->setTitle(QStringToTString(title));
		tag->setAlbum(QStringToTString(album));
		tag->setArtist(QStringToTString(artist));

		tag->setYear(year.toUInt());
		tag->setComment(QStringToTString(comment));
		tag->setTrack(track.toUInt());
		tag->setGenre(QStringToTString(genre));

		f.save();
	}
}

void frmMain::setTip()
{
	currentTrackFile.open(IO_WriteOnly);
	QTextStream streamOut(&currentTrackFile);
	streamOut.setEncoding(QTextStream::Unicode);

	QString title(QString::null),
		album(QString::null),
		artist(QString::null),
		bitrate(QString::null),
		length(QString::null),
		text(QString::null),
		pureText(QString::null);

	if (player.running())
	{
		pListIndex * p = collection.getPlaylist(playTab);

		if (!p)
			return;

		pList * l = collection.get(p, playIndex);

		if (!l)
			return;

		if (l->fileTags.tagTitle.isEmpty())
			title = l->trackName;
		else
			title = l->fileTags.tagTitle;
		
		text += ("<center><b><i>" + (QString)FALF_NAM + "</i></b><br></center>");
		text += ("<left><b>" + i18n("Title") + ":</b> " + title + "<br>");
		pureText += title;

		if (!l->fileTags.tagAlbum.isEmpty())
		{
			album = l->fileTags.tagAlbum;
			text += ("<left><b>" + i18n("Album") + ":</b> " + album + "<br>");
			pureText += (" - " + album);
		}

		if (!l->fileTags.tagArtist.isEmpty())
		{
			artist = l->fileTags.tagArtist;
			text += ("<left><b>" + i18n("Artist") + ":</b> " + artist + "<br>");
			pureText += (" - " + artist);
		}

		if (!l->fileTags.tagBitrate.isEmpty())
		{
			bitrate = l->fileTags.tagBitrate;
			text += ("<left><b>" + i18n("Bitrate") + ":</b> " + bitrate + " kbps");
			pureText += (" - " + bitrate + " kbps");
		}
	}
	else
		text = (QString)FALF_NAM;

	if (text != (QString)FALF_NAM)
	{
		if (pureText.isEmpty())
			pureText = text;

		setCaption((QString)FALF_NAM + " - [" + pureText + "]");

		streamOut << "TITLE: " << title
			<< "\nALBUM: "<< album
			<< "\nARTIST: " << artist
			<< "\nBITRATE: " << bitrate
			<< "\n" + (QString)FALF_NAM + " " + (QString)FALF_VER
			<< "\n";
	}
	else
	{
		setCaption((QString)FALF_NAM);
		streamOut << QString::null;
	}

	QToolTip::add(systemTray, text);

	currentTrackFile.close();
}

void frmMain::fileDropped(QDropEvent * event, QListViewItem * after)
{
	dropEvent(event);
}

void frmMain::scrollVol(QWheelEvent * e)
{
	unsigned int curVol = sldVol->value();

	if (e->delta() > 0)
		curVol += 5;
	else
		curVol -= 5;

	sldVol->setValue(curVol);
}

void frmMain::onMouseButtonReleased(const QPoint & point)
{
	pListIndex * p = collection.getPlaylist(tabPlaylist->currentPageIndex());

	if (!p)
		return;

	QListViewItem * c = lstPlaylist->currentItem();
	QListViewItem * i = lstPlaylist->itemAt(lstPlaylist->contentsToViewport(point));
	int itemIndex = lstPlaylist->itemIndex(i);
	int currentItemIndex = lstPlaylist->itemIndex(c);

	if (!i)
		return;

	if (c != i)
	{
		unsigned int indC = lstPlaylist->itemIndex(c);
		unsigned int indI = lstPlaylist->itemIndex(i);

		if (indC > indI)
		{
			for (unsigned int diff = 0 ; diff < (indC - indI) ; diff++)
				collection.up(p, indC - diff);
		}
		else
		{
			for (unsigned int diff = 0 ; diff < (indI - indC) ; diff++)
				collection.down(p, indC + diff);
		}

		randMove(currentItemIndex, itemIndex);

		refreshList();
		lstPlaylist->setCurrentItem(lstPlaylist->itemAtIndex(indI));
		ensureVisible();
	}
}

void frmMain::ensureVisible()
{
	QListViewItem * c = lstPlaylist->currentItem();

	if (!c)
		return;

	lstPlaylist->center(0, lstPlaylist->itemPos(c));
}

void frmMain::onStop()
{
	tabPlaylist->setCurrentPage(playTab);

	player.termThread();
	player.wait();
	playTimer.stop();
	lfmTimer.stop();
	sldPos->setValue(0);
	sldPos->setEnabled(false);
	pshAction->setPixmap(pixPlay);
	systemTray->setPixmap(pixStop);
	popupMenu->changeItem(0, pixPlay, i18n("Play"));
	action = STOP;
	stopAfter = false;

	randList.remove();
	refreshList();
	ensureVisible();

	setTip();
}

QString frmMain::browse(const QString & dir)
{
	QFileInfo info;
	QDir directory;
	QStringList files, dirs;
	QString results(QString::null);

	directory.setPath(dir);
	directory.setFilter(QDir::Files);
	directory.setNameFilter((filePattern + " " + filePattern.upper()));

	files = directory.entryList();

	QStringList::const_iterator i;

	for (i = files.begin() ; i != files.end() ; i++)
		results += ("file://" + dir + "/" + (*i));

	directory.setFilter(QDir::Dirs);
	directory.setNameFilter("*");

	dirs = directory.entryList();
	dirs.remove(".");
	dirs.remove("..");

	for (i = dirs.begin() ; i != dirs.end() ; i++)
		results += browse((dir + "/" + (*i)));

	return results;
}

void frmMain::store()
{
	pListIndex * p;
	pList * l;

	if (confFile.open(IO_WriteOnly))
	{
		QTextStream streamOut(&confFile);
		streamOut.setEncoding(QTextStream::Unicode);

		streamOut << "<files>" << collection.getCount() << "</files>\n" <<
			"<volume>" << sldVol->value() << "</volume>\n" <<
			"<x>" << x() << "</x>\n" <<
			"<y>" << y() << "</y>\n" <<
			"<loop>" << loop << "</loop>\n" <<
			"<rand>" << rnd << "</rand>\n" <<
			"<lfm>" << lfmOn << "</lfm>\n" <<
			"<user>" << lfmUser << "</user>\n" <<
			"<pass>" << lfmPassword << "</pass>\n";

		int empty = 0;
		for (int i = 0 ; i < tabPlaylist->count() ; i++)
		{
			p = collection.getPlaylist(i);

			if (!p)
				continue;

			l = p->list;

			if (!l)
			{
				if (i < playTab)
					empty++;
				continue;
			}

			streamOut << "<playlist>" << collection.getPlaylistName(p) << "</playlist>\n"
				<< "<descr>" << collection.getPlaylistDescr(p) << "</descr>\n" << "<begin>\n";

			while (l)
			{
				streamOut << "\t<file>" << l->filePath << "</file>\n";
				l = l->next;
			}

			streamOut << "<end>\n";
		}

		streamOut << "<page>" << (playTab - empty) << "</page>\n" <<
			"<track>" << playIndex << "</track>\n" <<
			"<frame>" << sldPos->value() << "</frame>\n" <<
			"<browse>" << browseDir << "</browse>\n" <<
			"<cddevice>" << audioCDDevice << "</cddevice>\n" <<
			"<codec>" << dlgLyrics.cmbCodec->currentItem() << "</codec>\n" <<
			"<sound>" << dlgSoundSystem.cmbSoundSystem->currentItem() << "</sound>\n";
	}

	confFile.close();

	lstPlaylist->saveLayout(KGlobal::config(), "KMListView lstPlaylist");
}

void frmMain::addFiles()
{
	QStringList files = KFileDialog::getOpenFileNames(browseDir,
		(filePattern + "|" + i18n("Audio files")), this, i18n("Add file(s)"));
	QStringList::const_iterator i = files.begin();

	if (i == files.end())
		return;

	QFileInfo fileInfo((*i));
	browseDir = fileInfo.dirPath(true);

	QString results(QString::null);

	for (; i != files.end() ; i++)
		results += ("file://" + (*i));

	if (!results.isEmpty())
		parse(results);
}

void frmMain::addFolders()
{
	KDirSelectDialog dlg(browseDir, true, this, 0, true);
	dlg.setCaption(i18n("Add folder"));
	
	if (dlg.exec() == QDialog::Rejected)
		return;

	QString dirName = dlg.url().path();
	QDir dir(dirName);

	if (!dirName.isEmpty())
		parse(dirName, true);

	if (!dir.cdUp())
		return;

	browseDir = dir.absPath();
}

void frmMain::saveM3u()
{
	if (!lstPlaylist->childCount())
		return;

	QString file = KFileDialog::getSaveFileName(browseDir, "*.m3u", this, i18n("Save playlist"));
	
	if (file.isEmpty())
		return;

	QFileInfo fileInfo(file);
	browseDir = fileInfo.dirPath(true);

	QFile fout(file);
	fout.open(IO_WriteOnly);

	QTextStream out(&fout);

	pListIndex * p = collection.getPlaylist(tabPlaylist->currentPageIndex());

	if (!p)
		return;

	pList * l = 0;
	unsigned int i = 0;

	while ((l = collection.get(p, i++)))
		out << l->filePath << "\n";

	fout.close();
}

void frmMain::openM3u(const QString & file)
{
	QFile fin(file);
	fin.open(IO_ReadOnly);

	QTextStream in(&fin);

	QString files;
	QString dir = file.left(file.findRev("/") + 1);

	while (!fin.atEnd())
	{
		QString tmp = in.readLine();
		int len = 0;

		for (unsigned int i = 0 ; i != tmp.length() ; i++)
		{
			if (tmp.at(i) != '#')
				len++;
			else
			{
				while ((i-- != 0))
				{
					if (tmp.at(i) == ' ')
						len--;
					else
						break;
				}

				break;
			}
		}

		if (len != 0)
		{
			if ((tmp = tmp.left(len)).at(0) != '/')
				tmp = (dir + tmp);

			files += ("file://" + tmp);
		}
	}

	fin.close();

	parse(files, true);
}

void frmMain::parse(const QString & data, bool ignoreM3u)
{
	int index = tabPlaylist->currentPageIndex();

	if (index < 0)
		return;

	pListIndex * p = collection.getPlaylist(index);

	if (!p)
		return;

	WAIT_CURSOR
	
	QStringList files = QStringList::split("file://", data);

	frmRestore dlgParse;
	dlgParse.setCaption(i18n("Adding files") + "...");
	dlgParse.prgRestore->setProgress(0);
	dlgParse.prgRestore->setTotalSteps(files.count());

	if (files.count() > 5)
		dlgParse.show();

	for (QStringList::const_iterator i = files.begin() ; i != files.end() ; i++)
	{
		QString file = (*i);
		file = file.stripWhiteSpace();
		file = KURL::decode_string(file);

		QString name = file.right(file.length() - file.findRev("/") - 1);

		QFileInfo info(file);

		if (info.isFile())
		{
			QString ext3 = file.right(4);
			
			if (filePattern.find(ext3, 0, false) != -1)
			{
				if (ext3.lower() != ".m3u")
				{
					exTags t = readTags(file);
					collection.add(p, name, file, &t.basic);
				}
				else
				{
					if (!ignoreM3u)
						openM3u(file);
				}
			}
		}
		else
		{
			if (info.isDir())
			{
				QString res = browse(file);
				parse(res, true);
			}
		}

		dlgParse.prgRestore->advance(1);
	}

	ARROW_CURSOR

	onTabChanged();
	setTabTip();
}

void frmMain::onSldPosValChanged()
{
	if ((!player.running()) || (!sldPos->maxValue()))
		return;

	QImage tmpImage = pixPlay.convertToImage();
	QPixmap tmpPixmap, dstPixmap = pixPlay;

	KIconEffect::toGamma(tmpImage, 1.0);
	tmpPixmap.convertFromImage(tmpImage);

	float
		percent = ((sldPos->value() * 100.0) / sldPos->maxValue()),
		toCpy = (dstPixmap.height() - (dstPixmap.height() * (percent / 100.0)));

	copyBlt(&tmpPixmap, 0, (int)toCpy, &dstPixmap, 0, (int)toCpy);

	systemTray->setPixmap(tmpPixmap);
}

int frmMain::getRandom()
{
	int index, c = lstPlaylist->childCount(), add = 0, sub = 0;
	
	if (randList.getCount() == c)
		randList.remove();
	else
	{
		int i;

		for (i = 0 ; i != (c - 1) ; i++)
		{
			if (randList.searchForIndex(i))
				add++;
			else
				break;
		}

		for (i = (c - 1) ; i != 0 ; i--)
		{
			if (randList.searchForIndex(i))
				sub++;
			else
				break;
		}
	}

	do
		index = ((rand() % (c - sub - add)) + add);
	while (randList.searchForIndex(index));

	return index;
}

void frmMain::random()
{
	action = STOP;

	if ((!loop) && (randList.getCount() == lstPlaylist->childCount()))
	{
		player.termThread();
		player.wait();
		playTimer.stop();
		sldPos->setValue(0);
		
		setTip();

		sldPos->setEnabled(false);
		pshAction->setPixmap(pixPlay);
		
		pshStop->setEnabled(false);
		popupMenu->setItemEnabled(3, false);
		systemTray->setPixmap(pixStop);
		popupMenu->changeItem(0, pixPlay, i18n("Play"));

		randList.remove();
	}
	else
	{
		lstPlaylist->setCurrentItem(lstPlaylist->itemAtIndex(getRandom()));
		
		onAction();
	}

	setStatBarMsg();
}

void frmMain::onRnd()
{
	if (rnd = !rnd)
	{
		pshRnd->setPixmap(pixRndOn);
		QToolTip::add(pshRnd, i18n("Turn off random mode"));

		if ((player.running()) && (!randList.searchForIndex(playIndex)))
			randList.addIndex(playIndex);
	}
	else
	{
		pshRnd->setPixmap(pixRndOff);
		QToolTip::add(pshRnd, i18n("Turn on random mode"));
		randList.remove();
	}
}

void frmMain::randMove(const int & current, const int & newCurrent)
{
	int add;

	if (current < newCurrent)
		add = 1;
	else
		add = -1;

	if (tabPlaylist->currentPageIndex() == playTab)
	{
		if (playIndex == newCurrent)
			playIndex = (newCurrent - add);
		else
		{
			if (playIndex == current)
				playIndex = newCurrent;
			else
			{
				if (((playIndex > current) && (playIndex < newCurrent)) ||
					((playIndex < current) && (playIndex > newCurrent)))
					playIndex -= add;
			}
		}
		
	}
	else
		return;

	if (randList.isEmpty())
		return;

	bool currPresent = randList.searchForIndex(current);

	if (currPresent)
		randList.deleteIndex(current);

	for (int c = (current + add) ; c != newCurrent ; c += add)
	{
		if (randList.searchForIndex(c))
		{
			randList.deleteIndex(c);
			randList.addIndex(c - add);
		}
	}

	if (!randList.searchForIndex(newCurrent))
	{
		if (currPresent)
		{
			if (randList.searchForIndex(newCurrent - add))
				randList.deleteIndex(newCurrent - add);

			randList.addIndex(newCurrent);
		}
	}
	else
	{
		if (!randList.searchForIndex(newCurrent - add))
			randList.addIndex(newCurrent - add);

		if (!currPresent)
			randList.deleteIndex(newCurrent);
	}
}

void frmMain::onOff()
{
	for (int i = 1 ; i != 4 ; i++)
		popupMenu->setItemEnabled(i, false);

	pshAction->setEnabled(true);
	popupMenu->setItemEnabled(0, true);
	pshStop->setEnabled(false);
	pshLeft->setEnabled(false);
	pshRight->setEnabled(false);

	if (player.running())
	{
		pshStop->setEnabled(true);
		pshLeft->setEnabled(true);
		pshRight->setEnabled(true);
		popupMenu->setItemEnabled(1, true);
		popupMenu->setItemEnabled(2, true);
		popupMenu->setItemEnabled(3, true);
		pshAction->setPixmap(pixPause);
		popupMenu->changeItem(0, pixPause, i18n("Pause"));
	}
	else
	{
		pshAction->setPixmap(pixPlay);
		popupMenu->changeItem(0, pixPlay, i18n("Play"));
		
		if (action == STOP)
			systemTray->setPixmap(pixStop);
	}
	
	bool enable = true;
	if (!lstPlaylist->childCount())
		enable = false;

	if ((!enable) && (action != PLAY))
	{
		pshAction->setEnabled(false);
		popupMenu->setItemEnabled(0, false);
	}
	
	if (action == PAUSE)
		return;

	pshUp->setEnabled(enable);
	pshDown->setEnabled(enable);
	pshRemove->setEnabled(enable);
	pshClearList->setEnabled(enable);

	if (tabPlaylist->count() != 1)
		pshDeletePlaylist->setEnabled(true);
	else
		pshDeletePlaylist->setEnabled(false);

	markCurrent();
}

void frmMain::lyricsDownloaded(bool e)
{
	QString text;

	if (!e)
	{
		if ((text = lyrics.getFromBuffer(buffer)).isEmpty())
			text = lyricsErrStr;
	}
	else
	{
		text = lyricsErrStr;
		delete buffer;
		buffer = 0;
	}
	
	QTextCodec * codec = QTextCodec::codecForName(dlgLyrics.cmbCodec->currentText());

	if (!codec)
		return;

	QString unicodeString = codec->toUnicode(text.latin1());

	dlgLyrics.txtLyrics->setText(unicodeString);
}

void frmMain::onDelLyrics()
{
	onLyrics(true);
}

void frmMain::onCodecChanged(int i)
{
	onLyrics();
}

void frmMain::onLyrics(bool force)
{
	pListIndex * p = collection.getPlaylist(playTab);

	if (!p)
		return;

	pList * item = collection.get(p, playIndex);

	if (!item)
		return;
	
	if (http)
	{
		http->disconnect();
		http->abort();
	}
	else
		http = new QHttp("www.autolyrics.com");
	
	QString title = item->fileTags.tagTitle;
	QString artist = item->fileTags.tagArtist;

	if ((!title.isEmpty()) && (!artist.isEmpty()))
		dlgLyrics.setCaption(title + " - " + artist);
	else
		dlgLyrics.setCaption(QString::null);

	dlgLyrics.txtLyrics->setText(i18n("Looking for lyrics") + "...");

	if (!dlgLyrics.isVisible())
		dlgLyrics.show();

	if (title.isEmpty() || artist.isEmpty())
	{
		dlgLyrics.txtLyrics->setText(lyricsErrStr);
		return;
	}

	title.replace(" ", "_");
	artist.replace(" ", "_");

	lyrics.setTitle(title);
	lyrics.setArtist(artist);
	
	QString text;

	if ((!force) && !(text = lyrics.getFromLocal()).isEmpty())
	{
		QTextCodec * codec = QTextCodec::codecForName(dlgLyrics.cmbCodec->currentText());

		if (!codec)
			return;

		QString unicodeString = codec->toUnicode(text.latin1());

		dlgLyrics.txtLyrics->setText(unicodeString);
		return;
	}

	buffer = new QBuffer;
	
	if (!buffer)
		return;

	connect(http, SIGNAL(done(bool)), this, SLOT(lyricsDownloaded(bool)));
	
	QString url = "/lyrics/for/" + title + "/by/" + artist;
	url = KURL::encode_string(url);

	http->get(url, buffer);
}

void frmMain::onKeyPressed(QKeyEvent * e)
{
	QListViewItem * item = lstPlaylist->currentItem();

	int i = -1;

	if (item)
		i = lstPlaylist->itemIndex(item);

	switch (e->key())
	{
		case Qt::Key_Return:
			onPlaylist();
			return;
		case Qt::Key_Up:
			if (i > 0)
			{
				lstPlaylist->setSelected(lstPlaylist->itemAtIndex(i - 1), true);
				ensureVisible();
			}
			return;
		case Qt::Key_Down:
			if ((i != -1) && ((i + 1) < lstPlaylist->childCount()))
			{
				lstPlaylist->setSelected(lstPlaylist->itemAtIndex(i + 1), true);
				ensureVisible();
			}
			return;
		case Qt::Key_PageUp:
			if (i > 15)
				lstPlaylist->setSelected(lstPlaylist->itemAtIndex(i - 16), true);
			else
			{
				QListViewItem * i = lstPlaylist->firstChild();
				if (i)
					lstPlaylist->setSelected(i, true);
			}
			ensureVisible();
			return;
		case Qt::Key_PageDown:
			if ((i != -1) && ((i + 16) < lstPlaylist->childCount()))
				lstPlaylist->setSelected(lstPlaylist->itemAtIndex(i + 16), true);
			else
			{
				QListViewItem * i = lstPlaylist->lastItem();
				if (i)
					lstPlaylist->setSelected(i, true);
			}
			ensureVisible();
			return;
		case Qt::Key_F3:
			onSearch(false);
			return;
	}

	if (e->text().isEmpty() || !lstPlaylist->childCount())
		return;

	txtSearch->setFocus();
	txtSearch->setText(e->text());
	txtSearch->show();

	searchTimer.stop();
	searchTimer.start(1500);
}

void frmMain::onSearch(bool begin)
{
	QListViewItem * item = 0;
	static int next = 0;
	static int stop = lstPlaylist->childCount();
	bool again = false;

	if (txtSearch->text().isEmpty())
		return;

	if (begin)
	{
		next = 0;
		stop = lstPlaylist->childCount();
	}
	
	while ((item = lstPlaylist->itemAtIndex(next)))
	{
		if (item->text(0).find(txtSearch->text(), 0, false) != -1)
		{
			lstPlaylist->setSelected(item, true);
			
			if (++next == lstPlaylist->childCount())
				next = 0;

			break;
		}

		if (++next == stop)
		{
			stop = next;

			if (stop == lstPlaylist->childCount())
				next = 0;

			if (again)
				break;

			again = true;
		}
	}

	ensureVisible();
}

void frmMain::onSearchTimer()
{
	txtSearch->hide();
	searchTimer.stop();
}

void frmMain::onSearchTextChanged()
{
	searchTimer.stop();
	searchTimer.start(1500);

	onSearch(true);
}

void frmMain::showContextMenu(const QPoint & p)
{
	contextMenu->popup(p);
}

void frmMain::openAudioCD()
{
	int index = tabPlaylist->currentPageIndex();

	if (index < 0)
		return;

	pListIndex * p = collection.getPlaylist(index);

	if (!p)
		return;

	if (audioCDDevice.isEmpty())
	{
		audioCDDevice = "/dev/";
		dlgAudioCDDevice.txtDevice->setFocus();
	}
	else
		dlgAudioCDDevice.pshCancel->setFocus();

	dlgAudioCDDevice.txtDevice->setText(audioCDDevice);
	dlgAudioCDDevice.exec();

	if (!dlgAudioCDDevice.ok)
		return;

	audioCDDevice = dlgAudioCDDevice.txtDevice->text();

	QStringList contents = player.getAudioCDContents(audioCDDevice);

	WAIT_CURSOR

	frmRestore dlgParseCD;
	dlgParseCD.setCaption(i18n("Adding audio CD contents") + "...");
	dlgParseCD.prgRestore->setProgress(0);
	dlgParseCD.prgRestore->setTotalSteps(contents.count());

	if (contents.count() > 5)
		dlgParseCD.show();

	int counter = 1;
	for (QStringList::const_iterator i = contents.begin() ; i != contents.end() ; i++, counter++)
	{
		exTags tags = readTags((*i));
		collection.add(p, i18n("Track") + " " + QString::number(counter), (*i), &tags.basic);
		dlgParseCD.prgRestore->advance(1);
	}

	ARROW_CURSOR

	onTabChanged();
}

void frmMain::clearStop(bool remove)
{
	if (!player.running())
		pshStop->setEnabled(false);
	
	int c = tabPlaylist->currentPageIndex();

	if (playTab == c)
	{
		if (player.running())
		{
			action = STOP;
			playTimer.stop();
			player.termThread();
			player.wait();
			sldPos->setValue(0);
			sldPos->setEnabled(false);
		}

		playIndex = 0;

		QListViewItem * i = lstPlaylist->firstChild();

		if (i)
			lstPlaylist->setSelected(i, true);
	}
	else
	{
		if ((remove) && (c < playTab))
			playTab--;
	}
}

void frmMain::copyOnPlaylist(int i)
{
	pListIndex * p = collection.getPlaylist(tabPlaylist->currentPageIndex());

	if (!p)
		return;

	QListViewItem * item = lstPlaylist->currentItem();

	int curSel = -1;

	if (!item)
		return;

	curSel = lstPlaylist->itemIndex(item);

	pList * l = collection.get(p, curSel);

	if (!l)
		return;

	collection.add(collection.getPlaylist(i), l->trackName, l->filePath, &l->fileTags);

	setTabTip();
}

void frmMain::sendOnPlaylist(int i)
{
	copyOnPlaylist(i);
	onRemove();
}

void frmMain::copyPlaylist()
{
	static QTime lastCalled;

	if (lastCalled.restart() < 1000)
	{
		QMessageBox msg(i18n("Question"), i18n("Do You really wanna duplicate playlist?"), QMessageBox::Warning,
			QMessageBox::Yes, QMessageBox::No | QMessageBox::Default, QMessageBox::NoButton);

		if (msg.exec() != QMessageBox::Yes)
			return;
	}

	pListIndex * p = CURRENT_PLAYLIST;
	pListIndex * np = 0;

	if (!p)
		return;

	if (p->count == 0)
		return;
	else
		np = collection.newPlaylist(p->name + " [" + i18n("copy").upper() + "]", p->description);

	pList * l = 0;
	int i = 0;

	while ((l = collection.get(p, i++)))
		collection.add(np, l->trackName, l->filePath, &l->fileTags);

	refreshPlaylist(tabPlaylist->currentPageIndex());
}

void frmMain::appendPlaylist(int i)
{
	pListIndex * p = CURRENT_PLAYLIST;
	pListIndex * np = collection.getPlaylist(i);

	if ((!p) || (!np))
		return;

	if (p->count == 0)
		return;

	pList * l = 0;
	int j = 0;

	while ((l = collection.get(p, j++)))
		collection.add(np, l->trackName, l->filePath, &l->fileTags);

	setTabTip();
}

void frmMain::restoreVolume()
{
	const int & vol = player.getVolume();

	if (vol < desiredVolume)
	{
		if ((vol + 5) > desiredVolume)
			sldVol->setValue(desiredVolume);
		else
			player.setVolume(vol + 5);
	}
	else
		volumeTimer.stop();
}

void frmMain::makeUpPlaylist(int id)
{
	const int & page = tabPlaylist->currentPageIndex();

	pListIndex * p = collection.getPlaylist(page);

	if (!p)
		return;

	WAIT_CURSOR

	int i = -1;

	if (page == playTab)
		i = playIndex;

	switch (id)
	{
		case 0:
			i = collection.rmDup(p, i);
		break;
		case 1:
			i = collection.rmDead(p, i);
		break;
		case 3:
			makeUpPlaylist(1);
			makeUpPlaylist(0);
			return;
		break;
	}

	CRandTree tmp_rand(randList);

	if (page == playTab)
	{
		playIndex = i;
		randList.remove();
	}
	else
		tmp_rand.remove();

	pList * next = 0;
	i = 0;

	while ((next = collection.get(p, i++)))
	{
		if ((!tmp_rand.isEmpty()) && (tmp_rand.searchForIndex(next->myId->previous)))
			randList.addIndex(next->myId->current);

		delete next->myId;
		next->myId = 0;
	}

	ARROW_CURSOR

	onTabChanged();
	setTabTip();
}

void frmMain::lastFmHandShake()
{
	qDebug("lastFmHandShake()");

	if (!lfmServer)
		lfmServer = new QHttp("post.audioscrobbler.com");
	else
	{
		lfmServer->disconnect();
		lfmServer->abort();
	}

	connect(lfmServer, SIGNAL(done(bool)), this, SLOT(lastFmResolved(bool)));

	delete lfmBuffer;
	lfmBuffer = new QBuffer;
		
	lfmServer->get(
		QString("/?hs=true&p=1.1&c=%1&v=%2&u=%3")
		.arg("fap")
		.arg("0.1")
		.arg(lfmUser),
		lfmBuffer);
}

void frmMain::lastFmResolved(bool e)
{
	qDebug("lastFmResolved()");
	
	QString result((QString)lfmBuffer->buffer());
	
	delete lfmBuffer;
	lfmBuffer = 0;

	if (!result.startsWith("UPTODATE"))
		return;

	delete lfmAddr;
	delete lfmChallenge;
	
	lfmChallenge = new QString(result.section('\n', 1, 1));
	QString addr(result.section('\n', 2, 2));
	lfmAddr = new QString(addr.mid(7, addr.findRev(":") - 7));
}

void frmMain::lastFmQueue()
{
	qDebug("lastFmQueue()");

	if (!lfmOn)
		return;

	pListIndex * p = collection.getPlaylist(playTab);

	if (!p)
		return;

	pList * i = collection.get(p, playIndex);

	if (!i)
		return;

	const tags & itemTags = i->fileTags;

	if (itemTags.tagArtist.isEmpty() || itemTags.tagTitle.isEmpty() || itemTags.tagLength.isEmpty())
		return;

	lfmItem item(itemTags.tagArtist, itemTags.tagTitle, itemTags.tagAlbum,
		QString::number(itemTags.tagLengthSeconds),
		QDateTime::currentDateTime(Qt::UTC).toString("yyyy-MM-dd hh:mm:ss"));

	item.artist = KURL::encode_string(item.artist);
	item.title = KURL::encode_string(item.title);
	item.album = KURL::encode_string(item.album);

	queue.append(item);
	
	if ((lfmServer) && (lfmServer->currentId()))
		return;

	if ((!lfmAddr) || (!lfmChallenge))
	{
		lastFmHandShake();
		return;
	}

	QString addr("/protocol_1.1"), request(QString::null);
	request = ("u=" + lfmUser + "&s=");
	request += KMD5(KMD5(lfmPassword).hexDigest() + lfmChallenge->utf8()).hexDigest();

	unsigned short int num = 0;
	for (lfmItemList::iterator it = queue.begin() ; it != queue.end() ; it++, num++)
	{
		QString n = QString::number(num);
		request += "&a[" + n + "]=";
		request += (*it).artist;
		request += "&t[" + n + "]=";
		request += (*it).title;
		request += "&b[" + n + "]=";
		request += (*it).album;
		request += "&m[" + n + "]=";
		request += "&l[" + n + "]=";
		request += (*it).length;
		request += "&i[" + n + "]=";
		request += (*it).date;
	}
	
	if (!lfmSubmit)
		lfmSubmit = new QHttp((*lfmAddr));
	else
	{
		lfmSubmit->disconnect();
		lfmSubmit->abort();
		lfmSubmit->setHost((*lfmAddr));
	}

	QHttpRequestHeader header("POST", addr);
	header.setValue("Host", (*lfmAddr));
	header.setContentType("application/x-www-form-urlencoded");

	connect(lfmSubmit, SIGNAL(done(bool)), this, SLOT(lastFmSubmited(bool)));
	
	delete lfmBuffer;
	lfmBuffer = new QBuffer;

	lfmSubmit->request(header, request.utf8(), lfmBuffer);

	qDebug("sizeof(lastFmQueue): " + QString::number(queue.size()));
}

void frmMain::lastFmSubmited(bool e)
{
	qDebug("lastFmSubmited()");

	QString data((QString)lfmBuffer->buffer());

	delete lfmBuffer;
	lfmBuffer = 0;

	if ((!e) && (data.startsWith("OK")))
	{
		statBar->message("last.fm: " + i18n("information submitted successfully"));
		queue.clear();
	}
	else
	{
		statBar->message("last.fm: " + i18n("information submission failed"));

		if (queue.count() == 10)
			queue.remove(queue.begin());

		lastFmHandShake();
	}
}

void frmMain::editLfm()
{
	dlgLfm.ok = false;
	dlgLfm.txtUser->setText(lfmUser);
	dlgLfm.txtPassword->setText(lfmPassword);
	dlgLfm.chkLfm->setChecked(lfmOn);
	dlgLfm.txtUser->setFocus();
	dlgLfm.exec();
	
	if (dlgLfm.ok)
	{
		QString prevUser, prevPassword;

		prevUser = lfmUser;
		prevPassword = lfmPassword;

		lfmUser = dlgLfm.txtUser->text();
		lfmPassword = dlgLfm.txtPassword->text();
		
		if (!dlgLfm.chkLfm->isChecked())
		{
			lfmTimer.stop();
			queue.clear();

			if (lfmServer)
			{
				lfmServer->disconnect();
				lfmServer->abort();
				delete lfmServer;
				lfmServer = 0;
			}

			if (lfmSubmit)
			{
				lfmSubmit->disconnect();
				lfmSubmit->abort();
				delete lfmSubmit;
				lfmSubmit = 0;
			}
		}
		else
		{
			if ((lfmOn != dlgLfm.chkLfm->isChecked()) ||
				(prevUser != lfmUser) || (prevPassword != lfmPassword))
				lastFmHandShake();
		}

		lfmOn = dlgLfm.chkLfm->isChecked();
	}
}

void frmMain::markCurrent()
{
	if (tabPlaylist->currentPageIndex() != playTab)
		return;

	QListViewItem * item = lstPlaylist->itemAtIndex(playIndex);

	if (!item)
		return;

	int i = lstPlaylist->childCount();

	while (i--)
	{
		QListViewItem * tmp = lstPlaylist->itemAtIndex(i);

		if (!tmp)
			continue;

		tmp->setPixmap(0, 0);
	}

	if (player.running())
		item->setPixmap(0, pixPlay);

	lstPlaylist->center(0, lstPlaylist->itemPos(item));
}

void frmMain::sortPlaylist(int type)
{
	const int page = tabPlaylist->currentPageIndex();
	pListIndex * p = collection.getPlaylist(page);

	if (!p)
		return;

	bool order = true;

	if (type > 3)
	{
		type -= 4;
		order = false;
	}

	WAIT_CURSOR

	collection.sort(p, type, order);

	pList * l = 0;
	int k = 0, item = 0;
	bool found = false;

	CRandTree tmp_rand(randList);

	if (playTab == page)
		randList.remove();
	else
		tmp_rand.remove();

	while ((l = collection.get(p, k++)))
	{
		if (playTab == page)
		{
			if ((!found) && (l->myId->previous == playIndex))
			{
				found = true;
				playIndex = l->myId->current;
				item = playIndex;
			}

			if ((!tmp_rand.isEmpty()) && (tmp_rand.searchForIndex(l->myId->previous)))
				randList.addIndex(l->myId->current);
		}

		delete l->myId;
		l->myId = 0;
	}

	ARROW_CURSOR

	refreshList();
	
	lstPlaylist->setSelected(lstPlaylist->itemAtIndex(item), true);
}

void frmMain::onStopPressed()
{
	QTimer::singleShot(500, this, SLOT(onShowStopMenu()));
}

void frmMain::onShowStopMenu()
{
	if (pshStop->isDown())
	{
		stopMenu->exec(QCursor::pos());
		pshStop->setDown(false);
	}
}

void frmMain::onStopMenu(int i)
{
	if (i == 0)
		onStop();
	else
		stopAfter = true;
}

void frmMain::onShowPlaylistHeaderMenu(int i)
{
	headerMenu->exec(QCursor::pos());
}

void frmMain::onPlaylistHeaderMenu()
{
	ints checked, unchecked;

	int i = 0;
	for (chks::const_iterator it = headerMenuItems.begin() ; it != headerMenuItems.end() ; it++, i++)
	{
		if ((*it)->isChecked())
			checked.append(i);
		else
			unchecked.append(i);
	}

	if (checked.count() == 0)
	{
		i = 0;
		for (chks::const_iterator it = headerMenuItems.begin() ; it != headerMenuItems.end() ; it++, i++)
			(*it)->setChecked((bool)lstPlaylist->columnWidth(i));
		return;
	}

	for (ints::const_iterator it = checked.begin() ; it != checked.end() ; it++)
	{
		if (lstPlaylist->columnWidth((*it)) == 0)
		{
			lstPlaylist->setColumnWidth((*it), 125);
			lstPlaylist->header()->setResizeEnabled(true, (*it));
		}
	}

	for (ints::const_iterator it = unchecked.begin() ; it != unchecked.end() ; it++)
	{
		if (lstPlaylist->columnWidth((*it)) > 0)
		{
			lstPlaylist->hideColumn((*it));
			lstPlaylist->header()->setResizeEnabled(false, (*it));
		}
	}
}

void frmMain::setStatBarMsg()
{
	pListIndex * p = collection.getPlaylist(tabPlaylist->currentPageIndex());
	const length & total = collection.getTotalLength();

	QString len, totalLen;

	if (p)
	{
		len = QString("(%1:%2:%3)").arg(p->len.h, 2).arg(p->len.m, 2).arg(p->len.s, 2);
		len.replace(" ", "0");
		len = (" " + len);

		totalLen = QString("(%1:%2:%3)").arg(total.h, 2).arg(total.m, 2).arg(total.s, 2);
		totalLen.replace(" ", "0");
		totalLen = (" " + totalLen);
	}

	QString msg = (QString::number(lstPlaylist->childCount()) + " " +
		i18n("track(s) on currently selected playlist") + len);

	if ((randList.getCount() > 1) && (playTab == tabPlaylist->currentPageIndex()))
		msg += (" (" + QString::number((randList.getCount() - 1)) + " " + i18n("played randomly") + ")");

	msg += (", " + i18n("in total") + " " +
		QString::number(collection.getCount()) + " " + i18n("track(s) on") + " " +
		QString::number(tabPlaylist->count()) + " " + i18n("playlist(s)") + totalLen);

	statBar->message(msg);
}

void frmMain::addTmpFiles()
{
	if (!tmpFiles.exists())
		return;

	tmpFilesTimer.stop();

	tmpFiles.open(IO_ReadOnly);
	QTextStream tmp(&tmpFiles);
	tmp.setEncoding(QTextStream::Unicode);
		QString files = tmp.readLine();
	tmpFiles.close();
	tmpFiles.remove();

	if (files.isEmpty())
		return;

	QMessageBox msg(i18n("Question"), QString::number(files.contains("file://")) + " " + 
		i18n("file(s) to add") + ".\n" + i18n("Do You wanna create new playlist?"), QMessageBox::Warning,
		QMessageBox::Yes | QMessageBox::Default, QMessageBox::No, QMessageBox::NoButton);

	show();

	if (msg.exec() != QMessageBox::No)
	{
		collection.newPlaylist(
			QDate::currentDate(Qt::LocalTime).toString() + " " +
			QTime::currentTime(Qt::LocalTime).toString(),
			i18n("Double click to rename playlist"));
	
		refreshPlaylist(collection.getPlaylistCount() - 1);
	}
	
	parse(files);

	tmpFilesTimer.start(3000);
}

void frmMain::onSoundSystem()
{
	int index = dlgSoundSystem.cmbSoundSystem->currentItem();
	dlgSoundSystem.ok = false;
	dlgSoundSystem.cmbSoundSystem->setFocus();
	dlgSoundSystem.exec();

	if (dlgSoundSystem.ok)
	{
		if (index == dlgSoundSystem.cmbSoundSystem->currentItem())
			return;

		bool wasRunning = false;

		if (player.running())
		{
			onAction();
			wasRunning = true;
		}

		player.setSoundSystem(dlgSoundSystem.cmbSoundSystem->currentText().lower());

		if (wasRunning)
			onAction();

		lstPlaylist->setFocus();
	}
	else
		dlgSoundSystem.cmbSoundSystem->setCurrentItem(index);
}

void frmMain::onTestCanDecode(const QDragMoveEvent * e, bool & accept)
{
	accept = QTextDrag::canDecode(e);
}

void frmMain::onReceivedDropEvent(QWidget * w, QDropEvent * e)
{
	QString txt;
	QTextDrag::decode(e, txt);
	
	if ((!txt.isEmpty()) || (!e->source()))
		return;

	int oldIndex = tabPlaylist->indexOf(e->source()),
		newIndex = tabPlaylist->indexOf(w);

	playTab = collection.movePlaylist(oldIndex, newIndex, playTab);

	refreshPlaylist(newIndex);
}

void frmMain::onInitiateDrag(QWidget * w)
{
	QTextDrag * d = new QTextDrag(QString::null, tabPlaylist->currentPage());

	d->drag();
}

void frmMain::onTabDblClicked(QWidget * w)
{
	pListIndex * p = collection.getPlaylist(tabPlaylist->indexOf(w));

	if (!p)
		return;

	dlgPlaylist.ok = false;
	dlgPlaylist.txtPlaylist->setText(collection.getPlaylistName(p));
	dlgPlaylist.txtDescription->setText(collection.getPlaylistDescr(p));
	dlgPlaylist.txtPlaylist->setFocus();

	dlgPlaylist.exec();

	if (dlgPlaylist.ok)
	{
		if (!dlgPlaylist.txtPlaylist->text().isEmpty())
			collection.setPlaylistName(p, dlgPlaylist.txtPlaylist->text());
		else
			collection.setPlaylistName(p, i18n("Unnamed Playlist"));
		collection.setPlaylistDescr(p, dlgPlaylist.txtDescription->text());
		refreshPlaylist(tabPlaylist->currentPageIndex());
	}
}
