#ifndef PLAYER_H
#include "player.h"

#include <kurl.h>

CPlayer::CPlayer()
{
	xine = xine_new();

	if (!xine)
		return;

	xine_init(xine);
	ao_port = xine_open_audio_driver(xine, NULL, NULL);

	term = true;
	pos = 0;

	int ma, mi, su;
	xine_get_version(&ma, &mi, &su);
	xineVer = QString::number(ma) + "." + QString::number(mi) + "." + QString::number(su);
}

CPlayer::~CPlayer()
{
	xine_close_audio_driver(xine, ao_port);
	xine_exit(xine);
}

void CPlayer::setTrack(const QString & t)
{
	track = t;
}

QString CPlayer::getTrack() const
{
	return track;
}

QStringList CPlayer::getAudioCDContents(const QString & device) const
{
	QStringList urls;
	char ** url;
	int num;

	if (!device.isNull())
	{
        	xine_cfg_entry_t config;
        	xine_config_lookup_entry(xine, "input.cdda_device", &config);
        	config.str_value = (char *)device.latin1();
        	xine_config_update_entry(xine, &config);
	}
	else
		return urls;

	url = xine_get_autoplay_mrls(xine, "CD", &num);

	if (url)
	{
		for (int i = 0 ; i != num ; i++)
		{
			if (url[i])
				urls << KURL::decode_string(url[i]);
		}
	}

	return urls;
}

const xine_t * CPlayer::getXineHandle() const
{
	return xine;
}

QString CPlayer::getXineVer() const
{
	return xineVer;
}
/*
int64_t CPlayer::getCurrentVpts() const
{
	if ((!term) && (pos) && (xine_get_status(stream) == XINE_STATUS_PLAY))
		return xine_get_current_vpts(stream);
	else
		return 0;
}
*/
void CPlayer::setVolume(const int & vol)
{
	volume = vol;
}

int CPlayer::getLength() const
{
	return length;
}

int CPlayer::getVolume() const
{
	return volume;
}

int CPlayer::getPos() const
{
	return pos;
}

void CPlayer::setPos(const int & p)
{
	pos = p;
}

void CPlayer::setSoundSystem(const QString & system)
{
	xine_close_audio_driver(xine, ao_port);
	ao_port = xine_open_audio_driver(xine, system.latin1(), NULL);
}

void CPlayer::termThread()
{
	term = true;
}

void CPlayer::wakeThread()
{
	term = false;
}

void CPlayer::run()
{
	if (!xine)
	{
		qDebug("Couldn't init xine!");
		return;
	}

	stream = xine_stream_new(xine, ao_port, NULL);

	if (!xine_open(stream, (const char *)track.local8Bit()) || !xine_play(stream, 0, pos))
		goto cleanup;

	while ((xine_get_status(stream) == XINE_STATUS_PLAY) && (!term))
	{
		xine_set_param(stream, XINE_PARAM_AUDIO_VOLUME, volume);
		xine_get_pos_length(stream, NULL, &pos, &length);

		QThread::msleep(250);
	}

cleanup:

	xine_close(stream);
	xine_dispose(stream);

	pos = 0;
	track = QString::null;
}

#endif
