/**

	playlist.cpp

	Playlist's engine source file

**/

#ifndef PLAYLIST_H

#include <qfile.h>

#include "playlist.h"

CPlaylist::CPlaylist()
{
	mainIndex = NULL;
	mainGuard = NULL;

	count = 0;
	playlistCount = 0;
}

CPlaylist::~CPlaylist()
{
	while (getPlaylistCount())
		delPlaylist(getPlaylist(0));
}

pListIndex * CPlaylist::newPlaylist(const QString & name, const QString & descr)
{
	pListIndex * newLst = new pListIndex;

	if (!newLst)
		return NULL;

	newLst->name = name;
	newLst->description = descr;
	newLst->previous = mainGuard;
	
	if (!mainIndex)
	{
		mainIndex = newLst;
		mainGuard = mainIndex;
		mainGuard->previous = NULL;
	}

	if (mainGuard)
		mainGuard->next = newLst;

	mainGuard = newLst;

	playlistCount++;

	return newLst;
}

void CPlaylist::setPlaylistName(const pListIndex * const index, const QString & name)
{
	if (!index)
		return;

	index->name = name;
}

QString CPlaylist::getPlaylistName(const pListIndex * const index) const
{
	if (!index)
		return QString::null;

	return index->name;
}

void CPlaylist::setPlaylistDescr(const pListIndex * const index, const QString & descr)
{
	if (!index)
		return;

	index->description = descr;
}

QString CPlaylist::getPlaylistDescr(const pListIndex * const index) const
{
	if (!index)
		return QString::null;

	return index->description;
}

void CPlaylist::delPlaylist(const pListIndex * const index)
{
	if (!index)
		return;

	if (index->previous)
		index->previous->next = index->next;

	if (index->next)
		index->next->previous = index->previous;

	if (index == mainGuard)
		mainGuard = index->previous;

	if (index == mainIndex)
		mainIndex = index->next;

	del(index, -1);

	delete index;

	playlistCount--;
}

pListIndex * CPlaylist::getPlaylist(const unsigned int & which) const
{
	pListIndex * tmp = mainIndex;

	for (unsigned int i = 0 ; i < which ; i++)
	{
		if (tmp->next)
			tmp = tmp->next;
		else
			return NULL;
	}

	return tmp;
}

int CPlaylist::movePlaylist(const unsigned int & which, const unsigned int & where, const unsigned int & current)
{
	pListIndex * src = getPlaylist(which);
	pListIndex * dst = getPlaylist(where);

	int newCurrent = current;

	if ((!src) || (!dst) || (src == dst))
		return newCurrent;

	if (which < where)
	{
		if (src->previous)
			src->previous->next = src->next;

		if (src->next)
			src->next->previous = src->previous;

		if (mainIndex == src)
			mainIndex = src->next;

		if (mainGuard == dst)
			mainGuard = src;

		src->next = dst->next;
		src->previous = dst;

		if (dst->next)
			dst->next->previous =  src;

		dst->next = src;

		if ((current >= which) && (current <= where))
		{
			if (which == current)
				newCurrent = where;
			else
				newCurrent--;
		}
	}
	else
	{
		if (src->next)
			src->next->previous = src->previous;

		if (src->previous)
			src->previous->next = src->next;

		if (mainIndex == dst)
			mainIndex = src;

		if (mainGuard == src)
			mainGuard = src->previous;

		src->previous = dst->previous;
		src->next = dst;

		if (dst->previous)
			dst->previous->next = src;

		dst->previous = src;

		if ((current <= which) && (current >= where))
		{
			if (which == current)
				newCurrent = where;
			else
				newCurrent++;
		}
	}

	return newCurrent;
}

unsigned int CPlaylist::getCount() const
{
	return count;
}

int CPlaylist::getPlaylistCount() const
{
	return playlistCount;
}

length CPlaylist::getTotalLength() const
{
	return totalLen;
}

pList * CPlaylist::add(const pListIndex * const index, const QString & name, const QString & path,
			const tags * const ftags)
{
	if (!index)
		return NULL;

	pList * newItem = new pList;

	if (!newItem)
		return NULL;

	newItem->trackName = name;
	if (ftags)
		newItem->fileTags = *ftags;
	newItem->filePath = path;
	
	if (!index->list)
		index->list = newItem;
	else
	{
		pList * tmp = index->list;
		while (tmp->next)
			tmp = tmp->next;
		tmp->next = newItem;
		newItem->previous = tmp;
	}

	count++;
	index->count++;
	index->len += newItem->fileTags.tagLengthSeconds;	
	totalLen += newItem->fileTags.tagLengthSeconds;

	return newItem;
}

void CPlaylist::del(const pListIndex * const index, const int & which)
{
	if (!index)
		return;

	pList * tmp = index->list;

	if (!tmp)
		return;

	if (which < 0)
	{
		pList * temp = NULL;;

		while (tmp)
		{
			temp = tmp->next;
			
			index->len -= tmp->fileTags.tagLengthSeconds;
			totalLen -= tmp->fileTags.tagLengthSeconds;

			delete tmp->myId;
			delete tmp;

			tmp = temp;
			count--;
			index->count--;
		}

		index->list = NULL;
	}
	else
	{
		for (int i = 0 ; i < which ; i++)
		{
			if (tmp->next)
				tmp = tmp->next;
			else
				return;
		}

		if (tmp->previous)
			tmp->previous->next = tmp->next;

		if (tmp->next)
			tmp->next->previous = tmp->previous;

		if ( (!tmp->previous) && (!tmp->next) )
			index->list = NULL;
		else
		{
			if (tmp == index->list)
				index->list = tmp->next;
		}

		index->len -= tmp->fileTags.tagLengthSeconds;
		totalLen -= tmp->fileTags.tagLengthSeconds;

		delete tmp->myId;
		delete tmp;

		count--;
		index->count--;
	}
}

void CPlaylist::up(const pListIndex * const index, const unsigned int & which)
{
	if ( (!index) || (which < 1) )
		return;

	pList * tmp = index->list;
	pList * temp = NULL;

	if (!tmp)
		return;

	for (unsigned int i = 0 ; i < which ; i++)
	{
		if (tmp->next)
			tmp = tmp->next;
		else
			return;
	}

	if (tmp->next)
		tmp->next->previous = tmp->previous;

	if (tmp->previous)
	{
		temp = tmp->previous->previous;

		if (tmp->previous->previous)
			tmp->previous->previous->next = tmp;

		tmp->previous->next = tmp->next;
		tmp->previous->previous = tmp;
	}

	tmp->next = tmp->previous;
	tmp->previous = temp;

	if (!tmp->previous)
		index->list = tmp;
}

void CPlaylist::down(const pListIndex * const index, const unsigned int & which)
{
	if (!index)
		return;

	pList * tmp = index->list;
	pList * temp = NULL;

	if (!tmp)
		return;

	for (unsigned int i = 0 ; i < which ; i++)
	{
		if (tmp->next)
			tmp = tmp->next;
		else
			return;
	}

	if (tmp->previous)
		tmp->previous->next = tmp->next;

	if (tmp->next)
	{
		temp = tmp->next->next;

		if (tmp->next->next)
			tmp->next->next->previous = tmp;

		tmp->next->previous = tmp->previous;
		tmp->next->next = tmp;
	}

	if (!tmp->previous)
		index->list = tmp->next;

	tmp->previous = tmp->next;
	tmp->next = temp;
}

pList * CPlaylist::get(const pListIndex * const index, const unsigned int & which) const
{
	if (!index)
		return NULL;

	pList * tmp = index->list;

	for (unsigned int i = 0 ; i < which ; i++)
	{
		if (tmp->next)
			tmp = tmp->next;
		else
			return NULL;
	}

	return tmp;
}

int CPlaylist::assignId(const pListIndex * const index)
{
	pList * item = 0;
	int i = 0;

	while ((item = get(index, i)))
	{
		delete item->myId;
		item->myId = new id(i);
		i++;
	}

	return i;
}

int CPlaylist::rmDup(const pListIndex * const index, const int & pI)
{
	if (!index)
		return pI;

	int newPI = pI;

	pList * item = 0;
	int i = 0;

	if (!assignId(index))
		return pI;

	while ((item = get(index, i)))
	{
		pList * jtem = 0;
		int j = (i + 1);
		
		while ((jtem = get(index, j)))
		{
			if (item->filePath == jtem->filePath)
			{
				pList * tmp_item = 0;
				int tmp_k = j;

				if (newPI == j)
				{
					tmp_k = i;
					del(index, i--);
					item = jtem;
					newPI--;
				}
				else
				{
					del(index, j);

					if (newPI > j)
						newPI--;
				}

				while ((tmp_item = get(index, tmp_k)))
				{
					tmp_item->myId->current = tmp_k;
					tmp_k++;
				}
			}
			else
				j++;
		}

		i++;
	}

	return newPI;
}

int CPlaylist::rmDead(const pListIndex * const index, const int & pI)
{
	if (!index)
		return pI;

	int newPI = pI;

	pList * item = 0;
	int i = 0;

	if (!assignId(index))
		return pI;

	while ((item = get(index, i)))
	{
		if (((item->filePath.left(6) != "cdda:/")) &&
			(!QFile::exists(item->filePath)))
		{
			pList * tmp_item = 0;
			int tmp_k = i;

			del(index, i);

			if (newPI > i)
				newPI--;

			while ((tmp_item = get(index, tmp_k)))
			{
				tmp_item->myId->current = tmp_k;
				tmp_k++;
			}
		}
		else
			i++;
	}

	return newPI;
}

void CPlaylist::sort(const pListIndex * const index, const unsigned short int & type, const bool & order)
{
	if (!index)
		return;

	pList * cur = get(index, 0);
	pList * next = 0;

	int max = 0, real = 0;

	max = assignId(index);

	while ((max > 1) && (cur))
	{
		int i = real;
		int j = (real + 1);

		while ((next = get(index, j)))
		{
			const QString * l = 0, * r = 0;
			const int * lL = 0, * rL = 0;

			switch (type)
			{
				case SORT_TITLE:
					l = &cur->fileTags.tagTitle;
					r = &next->fileTags.tagTitle;

					if (l->isEmpty())
						l = &cur->trackName;

					if (r->isEmpty())
						r = &next->trackName;
				break;
				case SORT_ALBUM:
					l = &cur->fileTags.tagAlbum;
					r = &next->fileTags.tagAlbum;
				break;
				case SORT_ARTIST:
					l = &cur->fileTags.tagArtist;
					r = &next->fileTags.tagArtist;
				break;
				case SORT_LENGTH:
					lL = &cur->fileTags.tagLengthSeconds;
					rL = &next->fileTags.tagLengthSeconds;
				break;
			}

			bool swap = false;

			if (type != SORT_LENGTH)
			{
				if (order)
				{
					if ((*l) > (*r))
						swap = true;
				}
				else
				{
					if ((*l) < (*r))
						swap = true;
				}
			}
			else
			{
				if (order)
				{
					if ((*lL) > (*rL))
						swap = true;
				}
				else
				{
					if ((*lL) < (*rL))
						swap = true;
				}
			}
			
			if (swap)
			{
				cur->myId->current = j;
				next->myId->current = i;

				for (int k = 0 ; k != (j - i) ; k++)
					down(index, (i + k));

				for (int k = 0 ; k != (j - i - 1) ; k++)
					up(index, (j - k - 1));

				cur = get(index, i);
			}

			j++;
		}

		cur = get(index, ++real);
	}
}

#endif
