/**

	playlist.h

	Playlist's engine header file

**/

#ifndef PLAYLIST_H
#define PLAYLIST_H

#include <qstring.h>

struct tags
{
	tags() : tagLengthSeconds(0) {};

	QString tagTitle;
	QString tagAlbum;
	QString tagArtist;
	QString tagBitrate;
	QString tagLength;
	int tagLengthSeconds;
};

struct exTags
{
	tags basic;

	QString tagYear;
	QString tagComment;
	QString tagTrack;
	QString tagGenre;
};

struct id
{
	id(const int & pre) : previous(pre), current(pre) {};

	int previous;
	int current;
};

struct length
{
	length() : h(0), m(0), s(0) {};

	void operator+=(const unsigned int & sec)
	{
		unsigned short int total_s, tmp_m;
		
		total_s = (sec + s);
		
		s = (total_s % 60);
		tmp_m = (m + ((total_s - s) / 60));
		m = (tmp_m % 60);
		h += ((tmp_m - m) / 60);
	}

	void operator-=(const unsigned int & sec)
	{
		while (s < sec)
		{
			if (m == 0)
			{
				h--;
				m = 60;
			}

			m--;
			s += 60;
		}

		s -= sec;
	}

	unsigned int h;
	unsigned short int m;
	unsigned short int s;
};

struct pList
{
	pList()
	{
		myId = 0;
		previous = 0;
		next = 0;
	}

	QString trackName;
	QString filePath;
	tags fileTags;

	id * myId;

	pList * next;
	pList * previous;
};

struct pListIndex
{
	pListIndex()
	{
		count = 0;
		list = 0;
		previous = 0;
		next = 0;
	}

	mutable QString name;
	mutable QString description;
	mutable length len;
	mutable int count;

	mutable pList * list;

	pListIndex * next;
	pListIndex * previous;
};

class CPlaylist
{
	public:
		CPlaylist();
		~CPlaylist();

		pListIndex * newPlaylist(const QString & name, const QString & descr);
		void setPlaylistName(const pListIndex * const index, const QString & name);
		QString getPlaylistName(const pListIndex * const index) const;
		void setPlaylistDescr(const pListIndex * const index, const QString & descr);
		QString getPlaylistDescr(const pListIndex * const index) const;
		void delPlaylist(const pListIndex * const index);
		pListIndex * getPlaylist(const unsigned int & which) const;
		int movePlaylist(const unsigned int & which, const unsigned int & where, const unsigned int & current);
		unsigned int getCount() const;
		int getPlaylistCount() const;
		length getTotalLength() const;

		pList * add(const pListIndex * const index, const QString & name, const QString & path,
			const tags * const ftags);
		void del(const pListIndex * const index, const int & which);
		void up(const pListIndex * const index, const unsigned int & which);
		void down(const pListIndex * const index, const unsigned int & which);
		pList * get(const pListIndex * const index, const unsigned int & which) const;

		int rmDup(const pListIndex * const index, const int & pI);
		int rmDead(const pListIndex * const index, const int & pI);
		void sort(const pListIndex * const index, const unsigned short int & type, const bool & order);

	private:
		pListIndex * mainIndex;
		pListIndex * mainGuard;
		unsigned int count;
		int playlistCount;
		length totalLen;
		
		enum sortBy {SORT_TITLE = 0, SORT_ALBUM = 1, SORT_ARTIST = 2, SORT_LENGTH = 3};

		int assignId(const pListIndex * const index);
};

#endif
