/**

	randtree.cpp

	Random track selector engine source file

**/

#ifndef RANDTREE_H

#include "randtree.h"

CRandTree::CRandTree()
{
	root = 0;
	count = 0;
}

CRandTree::CRandTree(const CRandTree & t)
{
	root = 0;
	count = 0;

	int found = 0, i = 0;

	while (found != t.getCount())
	{
		if (t.searchForIndex(i))
		{
			addIndex(i);
			found++;
		}

		i++;
	}
}

CRandTree::~CRandTree()
{
}

void CRandTree::addIndex(const int & i)
{
	nod * nn = new nod(i);
	
	if (!nn)
		return;

	count++;

	if (!root)
		root = nn;
	else
		return add(nn, root);
}

void CRandTree::add(const nod * const n, const nod * const r)
{
	if ((!n) || (!r))
		return;

	if (n->index < r->index)
	{
		if (r->left)
			add(n, r->left);
		else
			r->left = (nod *)n;
	}
	else
	{
		if (r->right)
			add(n, r->right);
		else
			r->right = (nod *)n;
	}
}

bool CRandTree::searchForIndex(const int & i) const
{
	return search(i, root);
}

bool CRandTree::search(const int & i, const nod * const r) const
{
	if (!r)
		return false;

	if (i != r->index)
	{
		if (i < r->index)
			return search(i, r->left);
		else
			return search(i, r->right);
	}
	else
		return true;
}

void CRandTree::deleteIndex(const int & i)
{
	del(i, root, 0);
	count--;
}

void CRandTree::del(const int & i, const nod * const r, const nod * const p)
{
	if (!r)
		return;

	if (i < r->index)
		del(i, r->left, r);
	else
	{
		if (i > r->index)
			del(i, r->right, r);
		else
		{
			if (p)
			{
				if (p->left == r)
					p->left = 0;
				else
					p->right = 0;

				add(r->left, root);
				add(r->right, root);

				delete r;
			}
			else
			{
				nod * tmp = root;

				if (root->left)
				{
					root = root->left;
					add(tmp->right, root);
				}
				else
				{
					if (root->right)
					{
						root = root->right;
						add(tmp->left, root);
					}
					else
						root = 0;
				}

				delete tmp;
			}
		}
	}
}

void CRandTree::remove(const nod * const r)
{
	if (r->left)
		remove(r->left);

	if (r->right)
		remove(r->right);

	delete r;
}

void CRandTree::remove()
{
	if (!root)
		return;

	remove(root);
	root = 0;
	count = 0;
}

bool CRandTree::isEmpty() const
{
	if (root)
		return false;
	else
		return true;
}

int CRandTree::getCount() const
{
	return count;
}

#endif
